"use strict";
/**
 * Components - Navigation
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Navigation = void 0;
const link_js_1 = require("../../utils/link/link.js");
const array_js_1 = require("../../utils/array/array.js");
const object_js_1 = require("../../utils/object/object.js");
const string_js_1 = require("../../utils/string/string.js");
const function_js_1 = require("../../utils/function/function.js");
const number_js_1 = require("../../utils/number/number.js");
const contentType_js_1 = require("../../utils/contentType/contentType.js");
const store_js_1 = require("../../store/store.js");
/**
 * Recursively generate navigation output
 */
class Navigation {
    /**
     * All navigations
     *
     * @type {Navigations[]}
     */
    navigations = [];
    /**
     * All navigation items
     *
     * @type {NavigationItem[]}
     */
    items = [];
    /**
     * Current link to compare against
     *
     * @type {string}
     */
    currentLink = '';
    /**
     * Current content type(s) to compare against
     *
     * @type {string[]}
     */
    currentType = [];
    /**
     * Initialize success
     *
     * @type {boolean}
     */
    init = false;
    /**
     * Navigation items by id
     *
     * @private
     * @type {NavigationItemsById}
     */
    #itemsById = new Map();
    /**
     * Navigations by location
     *
     * @private
     * @type {NavigationByLocation}
     */
    #navigationsByLocation = new Map();
    /**
     * Set properties and initialize
     *
     * @param {NavigationProps} props
     */
    constructor(props) {
        this.init = this.#initialize(props);
    }
    /**
     * Initialize - check required props and set props
     *
     * @private
     * @param {NavigationProps} props
     * @return {boolean}
     */
    #initialize(props) {
        /* Check props is object */
        if (!(0, object_js_1.isObjectStrict)(props)) {
            return false;
        }
        /* Defaults */
        const { navigations = [], items = [], currentLink = '', currentType = '' } = props;
        /* Check that required items exist */
        if (!(0, array_js_1.isArrayStrict)(navigations) || !(0, array_js_1.isArrayStrict)(items)) {
            return false;
        }
        /* Set variables */
        this.navigations = navigations;
        this.items = items;
        this.currentLink = currentLink;
        const typesArr = (0, array_js_1.isArrayStrict)(currentType) ? currentType : [currentType];
        this.currentType = typesArr.map(type => (0, contentType_js_1.normalizeContentType)(type));
        /* Items by id */
        this.items.forEach(item => {
            const info = this.#getItemInfo(item);
            if (info != null && (0, string_js_1.isStringStrict)(info.id)) {
                this.#itemsById.set(info.id, info);
            }
        });
        /* Navigations by location */
        this.navigations.forEach(nav => {
            if (!(0, object_js_1.isObjectStrict)(nav)) {
                return;
            }
            const { title = '', location = '', items = [] } = nav;
            const locations = (0, array_js_1.isArrayStrict)(location) ? location : [location];
            if (!(0, string_js_1.isStringStrict)(title) || !(0, array_js_1.isArrayStrict)(items)) {
                return;
            }
            locations.forEach(loc => {
                this.#navigationsByLocation.set(loc.toLowerCase().replace(/\s+/g, ''), {
                    title,
                    items
                });
            });
        });
        /* Init successful */
        return true;
    }
    /**
     * Normalize navigation item props
     *
     * @private
     * @param {NavigationItem} item
     * @return {NavigationItem|undefined}
     */
    #getItemInfo(item) {
        if (!(0, object_js_1.isObjectStrict)(item)) {
            return;
        }
        const { id, title = '', link, internalLink, externalLink = '', children } = item;
        let external = false;
        if ((0, string_js_1.isStringStrict)(externalLink)) {
            external = true;
        }
        const props = {
            id,
            title,
            link: (0, string_js_1.isStringStrict)(link) ? link : (0, link_js_1.getLink)(internalLink, externalLink),
            external
        };
        if ((0, string_js_1.isStringStrict)(props.link) && !external) {
            props.current = props.link === this.currentLink;
        }
        const internalId = internalLink?.id;
        if ((0, string_js_1.isStringStrict)(internalId)) {
            let isArchiveCurrent = false;
            for (const type of this.currentType) {
                const hasArchive = internalId === (0, store_js_1.getStoreItem)('archiveMeta')[type]?.id;
                if (hasArchive) {
                    isArchiveCurrent = true;
                    break;
                }
            }
            props.archiveCurrent = isArchiveCurrent;
        }
        let descendentCurrent = false;
        if ((0, array_js_1.isArrayStrict)(children)) {
            const storeChildren = [];
            descendentCurrent = this.#recurseItemChildren(children, storeChildren);
            props.children = storeChildren;
        }
        if (descendentCurrent) {
            props.descendentCurrent = descendentCurrent;
        }
        return {
            ...item,
            ...props
        };
    }
    /**
     * Check if any children match current link
     *
     * @private
     * @param {NavigationItem[]} children
     * @param {NavigationItem[]} output
     * @return {boolean}
     */
    #recurseItemChildren(children = [], output = []) {
        let childCurrent = false;
        children.forEach(child => {
            const info = this.#getItemInfo(this.#itemsById.get(child?.id));
            if (info == null) {
                return;
            }
            const { current = false, archiveCurrent = false } = info;
            if (current || archiveCurrent) {
                childCurrent = true;
            }
            output.push(info);
        });
        return childCurrent;
    }
    /**
     * Return navigation items by id
     *
     * @private
     * @param {NavigationItem[]} items
     * @return {NavigationItem[]}
     */
    #getItems(items = []) {
        const resItems = [];
        items.forEach(item => {
            if (!(0, object_js_1.isObjectStrict)(item)) {
                return;
            }
            const { id } = item;
            const storedItem = this.#itemsById.get(id);
            if (storedItem != null) {
                resItems.push(storedItem);
            }
        });
        return resItems;
    }
    /**
     * Loop through items to create html
     *
     * @private
     * @param {NavigationItem[]} items
     * @param {HtmlString} output
     * @param {NavigationOutputArgs} args
     * @param {number} depth
     * @param {number} maxDepth
     * @return {void}
     */
    #recurseOutput = (items = [], output, args, depth = 0, maxDepth) => {
        if ((0, number_js_1.isNumber)(maxDepth) && depth > maxDepth) {
            return;
        }
        const listFilterArgs = { args, output, items, depth };
        if ((0, function_js_1.isFunction)(args.filterBeforeList)) {
            args.filterBeforeList(listFilterArgs);
        }
        const listClasses = (0, string_js_1.isStringStrict)(args.listClass) ? ` class="${args.listClass}"` : '';
        const listAttrs = (0, string_js_1.isStringStrict)(args.listAttr) ? ` ${args.listAttr}` : '';
        output.html += `<ul data-nav-depth="${depth}"${listClasses}${listAttrs}>`;
        items.forEach((item, index) => {
            const { title = '', link = '', external = false, children = [], current = false, descendentCurrent = false, archiveCurrent = false } = item;
            /* Filters args */
            const filterArgs = { args, item, output, index, items, depth };
            /* Item start */
            if ((0, function_js_1.isFunction)(args.filterBeforeItem)) {
                args.filterBeforeItem(filterArgs);
            }
            const itemClasses = (0, string_js_1.isStringStrict)(args.itemClass) ? ` class="${args.itemClass}"` : '';
            let itemAttrs = (0, string_js_1.isStringStrict)(args.itemAttr) ? ` ${args.itemAttr}` : '';
            if (current) {
                itemAttrs += ' data-nav-current';
            }
            if (descendentCurrent) {
                itemAttrs += ' data-nav-descendent-current';
            }
            if (archiveCurrent) {
                itemAttrs += ' data-nav-archive-current';
            }
            output.html += `<li data-nav-depth="${depth}"${itemClasses}${itemAttrs}>`;
            /* Link start */
            if ((0, function_js_1.isFunction)(args.filterBeforeLink)) {
                args.filterBeforeLink(filterArgs);
            }
            const linkClassesArr = [];
            if ((0, string_js_1.isStringStrict)(args.linkClass)) {
                linkClassesArr.push(args.linkClass);
            }
            if (!external && (0, string_js_1.isStringStrict)(args.internalLinkClass)) {
                linkClassesArr.push(args.internalLinkClass);
            }
            const hasLink = (0, string_js_1.isStringStrict)(link);
            const linkClasses = (linkClassesArr.length > 0) ? ` class="${linkClassesArr.join(' ')}"` : '';
            const linkAttrsArr = [hasLink ? `href="${link}"` : 'type="button"'];
            if ((0, string_js_1.isStringStrict)(args.linkAttr)) {
                linkAttrsArr.push(args.linkAttr);
            }
            if (current) {
                linkAttrsArr.push('data-nav-current');
                if (hasLink) {
                    linkAttrsArr.push('aria-current="page"');
                }
            }
            if (descendentCurrent) {
                linkAttrsArr.push('data-nav-descendent-current');
            }
            if (archiveCurrent) {
                linkAttrsArr.push('data-nav-archive-current');
            }
            const linkAttrs = ` ${linkAttrsArr.join(' ')}`;
            const linkTag = hasLink ? 'a' : 'button';
            output.html += `<${linkTag} data-nav-depth="${depth}"${linkClasses}${linkAttrs}>`;
            if ((0, function_js_1.isFunction)(args.filterBeforeLinkText)) {
                args.filterBeforeLinkText(filterArgs);
            }
            output.html += title;
            if ((0, function_js_1.isFunction)(args.filterAfterLinkText)) {
                args.filterAfterLinkText(filterArgs);
            }
            /* Link end */
            output.html += `</${linkTag}>`;
            if ((0, function_js_1.isFunction)(args.filterAfterLink)) {
                args.filterAfterLink(filterArgs);
            }
            /* Nested content */
            if (children.length > 0) {
                this.#recurseOutput(children, output, args, depth + 1, maxDepth);
            }
            /* Item end */
            output.html += '</li>';
            if ((0, function_js_1.isFunction)(args.filterAfterItem)) {
                args.filterAfterItem(filterArgs);
            }
        });
        output.html += '</ul>';
        if ((0, function_js_1.isFunction)(args.filterAfterList)) {
            args.filterAfterList(listFilterArgs);
        }
    };
    /**
     * Return navigation html output
     *
     * @param {string} location
     * @param {NavigationOutputArgs} args
     * @param {number} maxDepth
     * @return {string} HTMLUListElement
     */
    getOutput(location = '', args, maxDepth) {
        const nav = this.#navigationsByLocation.get(location);
        if (nav == null) {
            return '';
        }
        const items = nav.items;
        const normalizedItems = this.#getItems(items);
        if (normalizedItems.length === 0) {
            return '';
        }
        args = Object.assign({
            listClass: '',
            listAttr: '',
            itemClass: '',
            itemAttr: '',
            linkClass: '',
            internalLinkClass: '',
            linkAttr: '',
            filterBeforeList: () => { },
            filterAfterList: () => { },
            filterBeforeItem: () => { },
            filterAfterItem: () => { },
            filterBeforeLink: () => { },
            filterAfterLink: () => { },
            filterBeforeLinkText: () => { },
            filterAfterLinkText: () => { }
        }, (0, object_js_1.isObjectStrict)(args) ? args : {});
        const output = { html: '' };
        this.#recurseOutput(normalizedItems, output, args, 0, maxDepth);
        return output.html;
    }
    /**
     * Return breadcrumbs html output
     *
     * @param {NavigationBreadcrumbItem[]} items
     * @param {string} current
     * @param {NavigationBreadcrumbOutputArgs} [args]
     * @return {string} HTMLOListElement
     */
    getBreadcrumbs(items, current = '', args) {
        /* Items required */
        if (!(0, array_js_1.isArrayStrict)(items)) {
            return '';
        }
        /* Args defaults */
        args = Object.assign({
            listClass: '',
            listAttr: '',
            itemClass: '',
            itemAttr: '',
            linkClass: '',
            internalLinkClass: '',
            linkAttr: '',
            currentClass: '',
            a11yClass: 'a-hide-vis',
            filterBeforeLink: () => { },
            filterAfterLink: () => { }
        }, (0, object_js_1.isObjectStrict)(args) ? args : {});
        /* List attributes */
        const listClasses = (0, string_js_1.isStringStrict)(args.listClass) ? ` class="${args.listClass}"` : '';
        const listAttrs = (0, string_js_1.isStringStrict)(args.listAttr) ? ` ${args.listAttr}` : '';
        /* Remove items that do not have title or slug */
        const filteredItems = items.filter((item) => {
            if (!(0, string_js_1.isStringStrict)(item.title)) {
                return false;
            }
            if (!(0, string_js_1.isString)(item.slug)) {
                return false;
            }
            item.slug = (0, link_js_1.getSlug)({
                id: item.id,
                slug: item.slug,
                contentType: item.contentType,
                pageData: item.internalLink
            });
            return true;
        });
        /* Loop through items */
        const itemClasses = (0, string_js_1.isStringStrict)(args.itemClass) ? ` class="${args.itemClass}"` : '';
        const itemAttrs = (0, string_js_1.isStringStrict)(args.itemAttr) ? ` ${args.itemAttr}` : '';
        const lastItemIndex = filteredItems.length - 1;
        const itemsArr = filteredItems.map((item, index) => {
            const { title, slug } = item;
            /* Link */
            const link = (0, link_js_1.getPermalink)(slug);
            /* Output store */
            const output = { html: '' };
            /* Check if last */
            const isLastLevel = lastItemIndex === index;
            /* Filter args */
            const filterArgs = { output, isLastLevel };
            /* Item */
            output.html += `<li${itemClasses}${itemAttrs}${isLastLevel ? ' data-nav-last' : ''}>`;
            /* Link */
            if ((0, function_js_1.isFunction)(args.filterBeforeLink)) {
                args.filterBeforeLink(filterArgs);
            }
            const linkClassesArr = [];
            if ((0, string_js_1.isStringStrict)(args.linkClass)) {
                linkClassesArr.push(args.linkClass);
            }
            if ((0, string_js_1.isStringStrict)(args.internalLinkClass)) {
                linkClassesArr.push(args.internalLinkClass);
            }
            const linkClasses = (linkClassesArr.length > 0) ? ` class="${linkClassesArr.join(' ')}"` : '';
            const linkAttrs = (0, string_js_1.isStringStrict)(args.linkAttr) ? ` ${args.linkAttr}` : '';
            output.html += `<a${linkClasses} href="${link}"${linkAttrs}>${title}</a>`;
            if ((0, function_js_1.isFunction)(args.filterAfterLink)) {
                args.filterAfterLink(filterArgs);
            }
            /* Close item */
            output.html += '</li>';
            return output.html;
        });
        /* Output */
        const currentClasses = (0, string_js_1.isStringStrict)(args.currentClass) ? ` class="${args.currentClass}"` : '';
        const a11yClasses = (0, string_js_1.isStringStrict)(args.a11yClass) ? ` class="${args.a11yClass}"` : '';
        return `
      <ol${listClasses}${listAttrs}>
        ${itemsArr.join('')}
        <li${itemClasses}${itemAttrs} data-nav-current>
          <span${currentClasses}>${current}<span${a11yClasses}> (current page)</span></span>
        </li>
      </ol>
    `;
    }
    /**
     * Return object of items stored by id
     *
     * @return {NavigationItemsById}
     */
    getItemsById() {
        return this.#itemsById;
    }
    /**
     * Return object of navigations stored by location
     *
     * @return {NavigationByLocation}
     */
    getNavigationsByLocation() {
        return this.#navigationsByLocation;
    }
}
exports.Navigation = Navigation;
