"use strict";
/**
 * Contentful - Data
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllContentfulData = exports.getContentfulData = void 0;
const contentful_resolve_response_1 = __importDefault(require("contentful-resolve-response"));
const ResponseError_js_1 = require("../utils/ResponseError/ResponseError.js");
const filter_js_1 = require("../utils/filter/filter.js");
const object_js_1 = require("../utils/object/object.js");
const string_js_1 = require("../utils/string/string.js");
const array_js_1 = require("../utils/array/array.js");
const config_js_1 = require("../config/config.js");
const store_js_1 = require("../store/store.js");
const contentfulDataNormal_js_1 = require("./contentfulDataNormal.js");
/**
 * Fetch data from contentful cms or cache
 *
 * @param {string} key
 * @param {ContentfulDataParams} params
 * @return {Promise<RenderItem[]>}
 */
const getContentfulData = async (key = '', params = {}) => {
    /* Key required for cache */
    if (!(0, string_js_1.isStringStrict)(key)) {
        throw new Error('No key');
    }
    /* Check cache */
    if (config_js_1.config.env.cache) {
        let cacheData = [];
        const cacheDataFilterArgs = {
            key,
            type: 'get'
        };
        cacheData = await (0, filter_js_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs, true);
        if ((0, object_js_1.isObject)(cacheData)) {
            return structuredClone(cacheData);
        }
    }
    /* Credentials */
    const { space, devCredential, prodCredential, devHost, prodHost } = config_js_1.config.cms;
    let accessToken = devCredential;
    let host = devHost;
    if (config_js_1.config.env.prod) {
        accessToken = prodCredential;
        host = prodHost;
    }
    if (!(0, string_js_1.isStringStrict)(space) || !(0, string_js_1.isStringStrict)(accessToken) || !(0, string_js_1.isStringStrict)(host)) {
        throw new Error('No credentials');
    }
    /* Params */
    let url = `https://${host}/spaces/${space}/environments/master/entries?access_token=${accessToken}`;
    for (const [key, value] of Object.entries(params)) {
        url += `&${key}=${value.toString()}`;
    }
    /* Check and transform data */
    const resp = await fetch(url);
    const data = await resp.json();
    if (!resp.ok) {
        throw new ResponseError_js_1.ResponseError('Bad fetch response', resp);
    }
    const resolvedData = (0, contentful_resolve_response_1.default)(data);
    const newData = (0, contentfulDataNormal_js_1.normalizeContentfulData)(resolvedData.items);
    /* Add to cache */
    if (config_js_1.config.env.cache) {
        const cacheDataFilterArgs = {
            key,
            type: 'set',
            data
        };
        await (0, filter_js_1.applyFilters)('cacheData', newData, cacheDataFilterArgs, true);
    }
    /* Output */
    return newData;
};
exports.getContentfulData = getContentfulData;
/**
 * Fetch data from all content types or single entry if serverless
 *
 * @param {AllContentfulDataArgs} args
 * @return {Promise<RenderAllData|undefined>}
 */
const getAllContentfulData = async (args = {}) => {
    /* All data */
    const { serverlessData, previewData } = args;
    let allData = {
        navigationItem: [],
        navigation: [],
        redirect: [],
        content: {
            page: []
        }
    };
    /* Serverless */
    const isServerless = serverlessData != null;
    const isPreview = previewData != null;
    const contentfulDataFilterArgs = {
        serverlessData,
        previewData
    };
    /* Get single entry data if serverless or preview data */
    let isEntry = false;
    if (isServerless || isPreview) {
        let contentType = '';
        let id = '';
        if (isServerless) {
            const slugs = await (0, store_js_1.fetchStoreItem)('slugs');
            const path = serverlessData.path;
            const item = slugs[path];
            if ((0, object_js_1.isObjectStrict)(item)) {
                id = (0, string_js_1.isStringStrict)(item.id) ? item.id : '';
                contentType = (0, string_js_1.isStringStrict)(item.contentType) ? item.contentType : '';
            }
        }
        if (isPreview) {
            id = previewData.id;
            contentType = previewData.contentType;
        }
        if (id !== '') {
            const key = `serverless_${id}_${contentType}`;
            const params = {
                'sys.id': id,
                include: 10
            };
            const data = await getContentfulData(key, params);
            if ((0, array_js_1.isArray)(data)) {
                isEntry = true;
                allData.content[contentType] = data;
            }
        }
    }
    /* Get partial data - not serverless */
    if (!isServerless || !isEntry) {
        const partial = config_js_1.config.partialTypes;
        for (const contentType of partial) {
            allData[contentType] = [];
            const key = `all_${contentType}`;
            const params = {
                content_type: contentType
            };
            let data = await getContentfulData(key, params);
            data = (0, filter_js_1.applyFilters)('contentfulData', data, contentfulDataFilterArgs);
            if ((0, array_js_1.isArray)(data)) {
                allData[contentType] = data;
            }
        }
    }
    /* Get whole data (for page generation) - not serverless or preview */
    if ((!isServerless && !isPreview) || !isEntry) {
        const whole = config_js_1.config.wholeTypes;
        for (const contentType of whole) {
            allData.content[contentType] = [];
            const key = `all_${contentType}`;
            const params = {
                content_type: contentType,
                include: 10
            };
            let data = await getContentfulData(key, params);
            data = (0, filter_js_1.applyFilters)('contentfulData', data, contentfulDataFilterArgs);
            if ((0, array_js_1.isArray)(data)) {
                allData.content[contentType] = data;
            }
        }
    }
    /* Filter all data */
    allData = (0, filter_js_1.applyFilters)('allData', allData, {
        type: 'contentful',
        serverlessData,
        previewData
    });
    /* Output */
    return allData;
};
exports.getAllContentfulData = getAllContentfulData;
