"use strict";
/**
 * Contentful - Data Normal
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeContentfulData = void 0;
const array_js_1 = require("../utils/array/array.js");
const object_js_1 = require("../utils/object/object.js");
const string_js_1 = require("../utils/string/string.js");
const objectUtils_js_1 = require("../utils/object/objectUtils.js");
const number_js_1 = require("../utils/number/number.js");
const config_js_1 = require("../config/config.js");
/**
 * Get html tag from type
 *
 * @private
 * @param {string} type
 * @return {string}
 */
const getTag = (type = 'text') => {
    const tags = {
        br: 'br',
        hr: 'hr',
        paragraph: 'p',
        blockquote: 'blockquote',
        bold: 'b',
        italic: 'i',
        underline: 'u',
        superscript: 'sup',
        subscript: 'sub',
        code: 'code',
        hyperlink: 'a',
        'entry-hyperlink': 'a',
        'asset-hyperlink': 'a',
        'heading-1': 'h1',
        'heading-2': 'h2',
        'heading-3': 'h3',
        'heading-4': 'h4',
        'heading-5': 'h5',
        'heading-6': 'h6',
        'list-item': 'li',
        'unordered-list': 'ul',
        'ordered-list': 'ol',
        table: 'table',
        'table-row': 'tr',
        'table-cell': 'td',
        'table-header-cell': 'th',
        text: ''
    };
    const tag = tags[type];
    return (0, string_js_1.isString)(tag) ? tag : '';
};
/**
 * Convert rich text to flatter props
 *
 * @private
 * @param {ContentfulDataItem[]} items
 * @return {RenderItem[]}
 */
const normalizeRichText = (items) => {
    const newItems = [];
    /* Recurse */
    items.forEach((item) => {
        if (!(0, object_js_1.isObjectStrict)(item)) {
            return;
        }
        const { nodeType, data, value, marks, content } = item;
        /* Tag */
        const tag = getTag(nodeType);
        /* Content */
        let contentValue;
        if ((0, string_js_1.isStringStrict)(value)) {
            contentValue = value;
            contentValue = contentValue.replace(/\n/g, '<br>');
            if ((0, array_js_1.isArrayStrict)(marks)) {
                const markTags = marks.map((m) => {
                    return getTag(m.type);
                });
                const markStart = markTags.map(m => `<${m}>`).join('');
                const markEnd = markTags.map(m => `</${m}>`).join('');
                contentValue = `${markStart}${value}${markEnd}`;
            }
        }
        if ((0, string_js_1.isStringStrict)(content)) {
            contentValue = content;
        }
        if ((0, array_js_1.isArrayStrict)(content)) {
            const contentArr = normalizeRichText(content);
            if (contentArr.length > 0) {
                contentValue = contentArr;
            }
        }
        /* Link */
        let link;
        let internalLink;
        if ((0, object_js_1.isObjectStrict)(data)) {
            link = (0, string_js_1.isString)(data.uri) ? data.uri : '';
            if ((0, object_js_1.isObjectStrict)(data.target)) {
                const target = data.target;
                if (nodeType === 'entry-hyperlink') {
                    internalLink = target;
                }
                const url = target?.fields?.file?.url;
                if (nodeType === 'asset-hyperlink' && (0, string_js_1.isString)(url)) {
                    link = url;
                }
            }
        }
        /* Output */
        if (contentValue == null) {
            return;
        }
        const newItem = {
            content: contentValue
        };
        if ((0, string_js_1.isStringStrict)(tag)) {
            newItem.tag = tag;
        }
        if ((0, string_js_1.isStringStrict)(link)) {
            newItem.link = link;
        }
        if ((0, object_js_1.isObjectStrict)(internalLink)) {
            newItem.internalLink = normalizeItem(internalLink, [], true);
        }
        return newItems.push(newItem);
    });
    /* Output */
    return newItems;
};
/**
 * Flatten nested file props
 *
 * @private
 * @param {ContentfulDataFile} file
 * @param {ContentfulDataFields} fields
 * @return {RenderFile}
 */
const normalizeFile = (file, fields) => {
    const type = (0, string_js_1.isString)(file.contentType) ? file.contentType : '';
    const format = type !== '' ? type.split('/')[1] : '';
    return {
        url: (0, string_js_1.isString)(file.url) ? `https:${file.url}` : '',
        name: (0, string_js_1.isString)(file.fileName) ? file.fileName : '',
        alt: (0, string_js_1.isString)(fields.description) ? fields.description : '',
        width: (0, number_js_1.isNumber)(file.details?.image?.width) ? file.details.image.width : 0,
        height: (0, number_js_1.isNumber)(file.details?.image?.height) ? file.details.image.height : 0,
        size: (0, number_js_1.isNumber)(file.details?.size) ? file.details.size : 0,
        format: format === 'jpeg' ? 'jpg' : format,
        type
    };
};
/**
 * Transform item props to flatter structure
 *
 * @private
 * @param {ContentfulDataItem} item
 * @param {RenderItem[]} data
 * @param {boolean} [isInternalLink]
 * @return {RenderItem}
 */
const normalizeItem = (item, data, isInternalLink = false) => {
    let newItem = {};
    /* Item */
    const itemCopy = { ...item };
    /* Id */
    if ((0, string_js_1.isString)(itemCopy.sys?.id)) {
        newItem.id = itemCopy.sys.id;
    }
    /* Type */
    let type = '';
    if ((0, string_js_1.isString)(itemCopy.sys?.type)) {
        type = itemCopy.sys.type;
    }
    if ((0, string_js_1.isString)(itemCopy.sys?.contentType?.sys?.id)) {
        type = itemCopy.sys.contentType.sys.id;
    }
    if (type !== '' && type !== 'Link') {
        newItem.contentType = type;
        if ((0, string_js_1.isString)(config_js_1.config.renderTypes[type])) {
            newItem.renderType = config_js_1.config.renderTypes[type];
        }
    }
    /* Metadata tags */
    if ((0, array_js_1.isArrayStrict)(itemCopy?.metadata?.tags)) {
        newItem.metadata = {
            tags: itemCopy.metadata.tags.map((t) => {
                return {
                    id: (0, string_js_1.isString)(t.sys?.id) ? t.sys.id : '',
                    name: (0, string_js_1.isString)(t.sys?.name) ? t.sys.name : ''
                };
            })
        };
    }
    /* Fields and file */
    if ((0, object_js_1.isObjectStrict)(itemCopy.fields)) {
        const fields = itemCopy.fields;
        const file = fields.file;
        if ((0, object_js_1.isObjectStrict)(file)) {
            return Object.assign(normalizeFile(file, fields), newItem);
        }
        (0, objectUtils_js_1.getObjectKeys)(fields).forEach((prop) => {
            const field = fields[prop];
            if (prop === 'content' && isInternalLink) {
                return;
            }
            if ((0, object_js_1.isObjectStrict)(field)) {
                if (field.nodeType === 'document') {
                    const { content } = field;
                    if ((0, array_js_1.isArrayStrict)(content)) {
                        newItem[prop] = content.map((c) => {
                            return (0, object_js_1.isObjectStrict)(c) ? normalizeItem(c, data, isInternalLink) : c;
                        });
                    }
                }
                else {
                    newItem[prop] = normalizeItem(field, data, prop === 'internalLink');
                }
            }
            else if ((0, array_js_1.isArrayStrict)(field)) {
                newItem[prop] = field.map((f) => {
                    return (0, object_js_1.isObjectStrict)(f) ? normalizeItem(f, data, isInternalLink) : f;
                });
            }
            else {
                newItem[prop] = field;
            }
        });
    }
    /* Rich text/nested content */
    if ((0, string_js_1.isString)(itemCopy.nodeType)) {
        const nodeType = itemCopy.nodeType;
        if (nodeType === 'embedded-entry-block' || nodeType === 'embedded-asset-block') {
            const content = (0, object_js_1.isObjectStrict)(itemCopy?.data?.target) ? itemCopy.data.target : item;
            newItem = normalizeItem(content, data, isInternalLink);
        }
        else {
            newItem.renderType = 'richText';
            newItem.tag = getTag(itemCopy.nodeType);
            if ((0, string_js_1.isString)(itemCopy.content)) {
                newItem.content = itemCopy.content;
            }
            if ((0, array_js_1.isArrayStrict)(itemCopy.content)) {
                newItem.content = normalizeRichText(itemCopy.content);
            }
        }
    }
    /* Output */
    return newItem;
};
/**
 * Transform contentful data into simpler objects
 *
 * @param {ContentfulDataItem[]} data
 * @param {RenderItem[]} [_newData]
 * @return {RenderItem[]}
 */
const normalizeContentfulData = (data, _newData = []) => {
    if (!(0, array_js_1.isArrayStrict)(data)) {
        return [];
    }
    /* Recurse data */
    data.forEach((item) => {
        if (!(0, object_js_1.isObjectStrict)(item)) {
            return;
        }
        _newData.push(normalizeItem(item, _newData));
    });
    /* Output */
    return _newData;
};
exports.normalizeContentfulData = normalizeContentfulData;
