"use strict";
/**
 * Local - Data
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllLocalData = exports.getLocalData = void 0;
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const filter_js_1 = require("../utils/filter/filter.js");
const store_js_1 = require("../store/store.js");
const object_js_1 = require("../utils/object/object.js");
const string_js_1 = require("../utils/string/string.js");
const array_js_1 = require("../utils/array/array.js");
const json_js_1 = require("../utils/json/json.js");
const config_js_1 = require("../config/config.js");
const localDataNormal_js_1 = require("./localDataNormal.js");
/**
 * Get data from file or cache
 *
 * @param {string} key
 * @param {LocalDataParams} params
 * @return {Promise<LocalData | null>}
 */
const getLocalData = async (key = '', params = {}) => {
    /* Key required for cache */
    if (!(0, string_js_1.isStringStrict)(key)) {
        throw new Error('No key');
    }
    /* Check cache */
    if (config_js_1.config.env.cache) {
        let cacheData = {};
        const cacheDataFilterArgs = {
            key,
            type: 'get'
        };
        cacheData = await (0, filter_js_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs, true);
        if ((0, object_js_1.isObject)(cacheData)) {
            return structuredClone(cacheData);
        }
    }
    /* Directory */
    const dir = config_js_1.config.local.dir;
    /* Params */
    const { all = true, id = '' } = params;
    /* Data */
    const data = {};
    let hasData = false;
    /* Single file */
    if ((0, string_js_1.isStringStrict)(id) && !all) {
        const file = await (0, promises_1.readFile)((0, node_path_1.resolve)(dir, `${id}.json`), { encoding: 'utf8' });
        const fileJson = (0, json_js_1.getJson)(file);
        if (fileJson != null) {
            data[id] = fileJson;
            hasData = true;
        }
    }
    /* All files */
    if (!(0, string_js_1.isStringStrict)(id) && all) {
        const files = await (0, promises_1.readdir)((0, node_path_1.resolve)(dir));
        for (const file of files) {
            const fileExt = (0, node_path_1.extname)(file);
            const fileName = (0, node_path_1.basename)(file, fileExt);
            if (fileExt === '.json') {
                const fileContents = await (0, promises_1.readFile)((0, node_path_1.resolve)(dir, file), { encoding: 'utf8' });
                const fileJson = (0, json_js_1.getJson)(fileContents);
                if (fileJson != null) {
                    data[fileName] = fileJson;
                    hasData = true;
                }
            }
        }
    }
    /* Data not empty check */
    if (!hasData) {
        throw new Error('No file data');
    }
    /* Add to cache */
    if (config_js_1.config.env.cache) {
        const cacheDataFilterArgs = {
            key,
            type: 'set',
            data
        };
        await (0, filter_js_1.applyFilters)('cacheData', data, cacheDataFilterArgs, true);
    }
    /* Output */
    return data;
};
exports.getLocalData = getLocalData;
/**
 * Get data from file system
 *
 * @param {AllLocalDataArgs} args
 * @return {Promise<RenderAllData|undefined>}
 */
const getAllLocalData = async (args) => {
    /* Args */
    const { resolveProps = {
        image: ['image'],
        data: ['items', 'internalLink', 'parent']
    }, excludeProps = {
        data: ['content'],
        archive: {
            posts: ['content'],
            terms: ['content']
        }
    } } = args;
    /* Get data */
    let data = await getLocalData('all_file_data', { all: true });
    /* Data must be object */
    if (!(0, object_js_1.isObjectStrict)(data)) {
        throw new Error('No file data');
    }
    /* All data */
    let allData = {
        navigationItem: [],
        navigation: [],
        redirect: [],
        content: {
            page: []
        }
    };
    config_js_1.config.partialTypes.forEach((contentType) => {
        allData[contentType] = [];
    });
    config_js_1.config.wholeTypes.forEach((contentType) => {
        allData.content[contentType] = [];
    });
    /* Image data */
    const imageMeta = (0, store_js_1.fetchStoreItem)('imageMeta');
    (0, localDataNormal_js_1.resolveInternalLinks)(imageMeta, data, resolveProps.image);
    /* Set id */
    Object.entries(data).forEach(([key, item]) => {
        if (!(0, object_js_1.isObjectStrict)(item)) {
            return;
        }
        const { contentType } = item;
        if ((0, string_js_1.isStringStrict)(contentType)) {
            item.id = key;
        }
    });
    /* Internal props */
    resolveProps.data.forEach((d) => {
        (0, localDataNormal_js_1.resolveInternalLinks)(data, data, [d], (prop, value) => {
            if (resolveProps.data.includes(prop)) {
                const newValue = (0, localDataNormal_js_1.undefineProps)(value, excludeProps.data);
                return newValue;
            }
            return value;
        });
    });
    /* Filter data */
    data = (0, filter_js_1.applyFilters)('localData', data);
    /* Empty archive data */
    const archivePosts = {};
    // const archiveTerms: any = {}
    /* Set content */
    Object.entries(data).forEach(([, item]) => {
        if (!(0, object_js_1.isObjectStrict)(item)) {
            return;
        }
        const { contentType } = item;
        if (!(0, string_js_1.isStringStrict)(contentType)) {
            return;
        }
        const partial = allData[contentType];
        if ((0, array_js_1.isArray)(partial)) {
            partial.push(item);
        }
        const whole = allData.content[contentType];
        if ((0, array_js_1.isArray)(whole)) {
            whole.push(item);
        }
        /* Archive */
        if ((0, store_js_1.getStoreItem)('archiveMeta')[contentType] != null) {
            const dataItemCopy = (0, localDataNormal_js_1.undefineProps)(item, excludeProps.archive.posts);
            if (archivePosts[contentType] == null) {
                archivePosts[contentType] = [];
            }
            archivePosts[contentType].push(dataItemCopy);
        }
    });
    /* Term content */
    /*
    Object.keys(config.contentTypes.taxonomy).forEach((tax) => {
      const { contentTypes, props } = config.contentTypes.taxonomy[tax]
  
      if (archiveTerms.terms[tax] == null) {
        archiveTerms.terms[tax] = {}
      }
  
      contentTypes.forEach((ct, i) => {
        const contentData = allData.content[ct]
  
        if (isArrayStrict(contentData)) {
          if (archiveTerms.terms[tax][ct] == null) {
            archiveTerms.terms[tax][ct] = {}
          }
  
          contentData.forEach((cd) => {
            const prop = props[i]
            const terms = cd[prop] as Generic[]
            const dataCopy = undefineProps(cd, excludeProps.archive.terms)
  
            if (isArrayStrict(terms)) {
              terms.forEach((term) => {
                if (!isObjectStrict(term)) {
                  return
                }
  
                const termId = term.id
  
                if (!isStringStrict(termId)) {
                  return
                }
  
                if (archiveTerms[tax][ct][termId] == null) {
                  archiveTerms[tax][ct][termId] = []
                }
  
                const archiveTaxContentTypeTerm = archiveTerms[tax][ct][termId]
  
                if (isArray(archiveTaxContentTypeTerm)) {
                  archiveTaxContentTypeTerm.push(dataCopy)
                }
              })
            }
          })
        }
      })
    })
    */
    /* Filter all data */
    const allDataFilterArgs = {
        type: 'local'
    };
    allData = (0, filter_js_1.applyFilters)('allData', allData, allDataFilterArgs);
    /* Output */
    return allData;
};
exports.getAllLocalData = getAllLocalData;
