"use strict";
/**
 * Render
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.setRenderFunctions = exports.renderHttpError = exports.renderNavigations = exports.renderLayout = exports.renderFunctions = exports.renderContent = exports.renderItem = exports.render = void 0;
const action_js_1 = require("../utils/action/action.js");
const filter_js_1 = require("../utils/filter/filter.js");
const link_js_1 = require("../utils/link/link.js");
const string_js_1 = require("../utils/string/string.js");
const array_js_1 = require("../utils/array/array.js");
const object_js_1 = require("../utils/object/object.js");
const number_js_1 = require("../utils/number/number.js");
const function_js_1 = require("../utils/function/function.js");
const shortcode_js_1 = require("../utils/shortcode/shortcode.js");
const tag_js_1 = require("../utils/tag/tag.js");
const store_js_1 = require("../store/store.js");
const redirects_js_1 = require("../redirects/redirects.js");
const serverless_js_1 = require("../serverless/serverless.js");
const scriptStyle_js_1 = require("../utils/scriptStyle/scriptStyle.js");
const Container_js_1 = require("../layouts/Container/Container.js");
const Column_js_1 = require("../layouts/Column/Column.js");
const Form_js_1 = require("../objects/Form/Form.js");
const Field_js_1 = require("../objects/Field/Field.js");
const RichText_js_1 = require("../text/RichText/RichText.js");
/**
 * Default render functions
 *
 * @type {RenderFunctions}
 */
const defaultRenderFunctions = {
    container: Container_js_1.Container,
    column: Column_js_1.Column,
    form: Form_js_1.Form,
    field: Field_js_1.Field,
    richText: RichText_js_1.RichText
};
/**
 * Output elements in render content
 *
 * @type {RenderFunctions}
 */
let renderFunctions = {
    ...defaultRenderFunctions
};
exports.renderFunctions = renderFunctions;
/**
 * Output html element
 *
 * @type {RenderLayout}
 */
let renderLayout = () => '';
exports.renderLayout = renderLayout;
/**
 * Output http error page
 *
 * @type {RenderHttpError}
 */
let renderHttpError = () => '';
exports.renderHttpError = renderHttpError;
/**
 * Navigations output for use in render functions
 *
 * @type {RenderNavigations}
 */
let renderNavigations = () => ({});
exports.renderNavigations = renderNavigations;
/**
 * Set content, layout and navigation output functions
 *
 * @param {RenderFunctionsArgs} args
 * @return {boolean}
 */
const setRenderFunctions = (args) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return false;
    }
    const { functions, layout, navigations, httpError } = args;
    if (!(0, object_js_1.isObjectStrict)(functions) || !(0, function_js_1.isFunction)(layout)) {
        return false;
    }
    exports.renderFunctions = renderFunctions = {
        ...defaultRenderFunctions,
        ...functions
    };
    exports.renderLayout = renderLayout = layout;
    if ((0, function_js_1.isFunction)(navigations)) {
        exports.renderNavigations = renderNavigations = navigations;
    }
    if ((0, function_js_1.isFunction)(httpError)) {
        exports.renderHttpError = renderHttpError = httpError;
    }
    return true;
};
exports.setRenderFunctions = setRenderFunctions;
/**
 * Get content and templates in expected format to map
 *
 * @private
 * @param {RenderItem[]} content
 * @param {RenderItem[]} [_templates]
 * @return {RenderTemplate}
 */
const getContentTemplate = (content, _templates = []) => {
    /* Content must be array */
    if (!(0, array_js_1.isArrayStrict)(content)) {
        return {
            content: [],
            templates: _templates
        };
    }
    /* One level loop */
    const _content = content.map((c) => {
        /* Check if template */
        if ((0, tag_js_1.tagExists)(c, 'template')) {
            _templates.push(structuredClone(c));
            /* Replace template with template break */
            return {
                metadata: {
                    tags: [
                        {
                            id: 'templateBreak',
                            name: ''
                        }
                    ]
                }
            };
        }
        /* Output */
        return c;
    });
    /* Output */
    return {
        content: _content,
        templates: _templates
    };
};
/**
 * Map out content slots to templates
 *
 * @private
 * @param {RenderItem[]} templates
 * @param {RenderItem[]} [content]
 * @return {RenderItem[]}
 */
const mapContentTemplate = (templates, content = []) => {
    /* Templates must be arrays */
    if (!(0, array_js_1.isArrayStrict)(templates)) {
        return templates;
    }
    /* Recurse templates */
    const lastTemplateIndex = templates.length - 1;
    templates.forEach((t, i) => {
        /* Remove template break */
        if (content[0] != null && (0, tag_js_1.tagExists)(content[0], 'templateBreak') && content.length >= 1) {
            content.shift();
        }
        /* Check if optional - remove if no content */
        const isOptional = (0, tag_js_1.tagExists)(t, 'templateOptional');
        if (isOptional && i === lastTemplateIndex && content.length === 0) {
            templates.pop();
            return;
        }
        /* Check if slot */
        const isSlot = (0, tag_js_1.tagExists)(t, 'templateSlot');
        /* Check for repeatable template item */
        const children = t.content;
        if ((0, array_js_1.isArrayStrict)(children) && !isSlot) {
            let repeat;
            const repeatIndex = children.findIndex((c) => {
                const isRepeat = (0, tag_js_1.tagExists)(c, 'templateRepeat');
                if (isRepeat) {
                    repeat = c;
                }
                return isRepeat;
            });
            if (repeatIndex !== -1 && repeat != null) {
                let breakIndex = content.findIndex((c) => {
                    return (0, tag_js_1.tagExists)(c, 'templateBreak');
                });
                breakIndex = breakIndex === -1 ? content.length : breakIndex;
                let insertIndex = repeatIndex;
                for (let j = insertIndex; j < breakIndex - 1; j += 1) {
                    children.splice(insertIndex, 0, structuredClone(repeat));
                    insertIndex = j;
                }
            }
        }
        /* Replace slot with content */
        if (isSlot && content.length >= 1) {
            const fill = content.shift();
            if (fill != null) {
                templates[i] = fill;
            }
            return;
        }
        /* Recurse children */
        if ((0, array_js_1.isArray)(children) && templates[i] != null) {
            templates[i].content = mapContentTemplate(children, content);
        }
    });
    /* Output */
    return templates;
};
/**
 * Recurse and output nested content
 *
 * @param {RenderContentArgs} args
 * @param {HtmlString} [_output]
 * @return {Promise<string>}
 */
const renderContent = async (args, _output = { html: '' }) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return _output.html;
    }
    const { content = [], serverlessData, pageData, pageContains = [], pageHeadings = [], navigations = {} } = args;
    let { parents = [], headingsIndex = -1, depth = 0 } = args;
    /* Content must be array */
    if (!(0, array_js_1.isArrayStrict)(content)) {
        return _output.html;
    }
    /* Loop */
    for (const item of content) {
        /* Item must be object */
        if (!(0, object_js_1.isObjectStrict)(item)) {
            continue;
        }
        /* Render props */
        const props = structuredClone(item);
        const renderType = (0, string_js_1.isString)(props.renderType) ? props.renderType : '';
        const isRichText = renderType === 'richText';
        /* Check for nested content */
        let children = props.content;
        /* Map out content to template */
        if (renderType === 'contentTemplate') {
            const template = getContentTemplate((0, array_js_1.isArray)(props.content) ? props.content : []);
            const templates = mapContentTemplate(template.templates, template.content);
            children = templates;
        }
        /* Children check */
        let childrenArr;
        let childrenStr = '';
        if ((0, array_js_1.isArrayStrict)(children) && !isRichText) {
            childrenArr = children;
        }
        if ((0, string_js_1.isStringStrict)(children) && !isRichText) {
            childrenStr = children;
        }
        /* Rich text area headings */
        if (renderType === 'content' && depth === 0) {
            headingsIndex = pageHeadings.push([]) - 1;
        }
        /* Render output */
        let renderStart = '';
        let renderEnd = '';
        let filterType = '';
        let filterArgs = {};
        const renderFunction = renderFunctions[renderType];
        if ((0, function_js_1.isFunction)(renderFunction)) {
            const renderArgs = {
                args: props,
                parents,
                pageData,
                pageContains,
                navigations,
                serverlessData
            };
            if (childrenArr != null) {
                renderArgs.children = childrenArr;
            }
            if (renderType === 'richText') {
                renderArgs.headings = pageHeadings[headingsIndex];
            }
            let renderOutput = await renderFunction(renderArgs);
            if ((0, array_js_1.isArrayStrict)(renderOutput)) {
                if (renderOutput.length === 1) {
                    renderOutput = renderOutput[0]?.split('%content') ?? ['', ''];
                }
                const [start, end] = renderOutput;
                if ((0, string_js_1.isString)(start)) {
                    renderStart = start;
                }
                if ((0, string_js_1.isString)(end)) {
                    renderEnd = end;
                }
            }
            if ((0, string_js_1.isString)(renderOutput)) {
                renderStart = renderOutput;
            }
            pageContains.push(renderType);
            filterType = renderType;
            filterArgs = {
                ...props,
                content: undefined
            };
        }
        /* Filter content output */
        const renderContentFilterArgs = {
            renderType: filterType,
            args: filterArgs
        };
        const [filterRenderStart, filterRenderEnd] = await (0, filter_js_1.applyFilters)('renderContent', [renderStart, renderEnd], renderContentFilterArgs, true);
        if ((0, string_js_1.isString)(filterRenderStart)) {
            renderStart = filterRenderStart;
        }
        if ((0, string_js_1.isString)(filterRenderEnd)) {
            renderEnd = filterRenderEnd;
        }
        /* Add to output object */
        _output.html += renderStart + childrenStr;
        /* Recurse through children */
        if (childrenArr != null) {
            const parentsCopy = [...parents];
            parentsCopy.unshift({
                renderType,
                args: {
                    ...props,
                    content: undefined,
                    parents: undefined
                }
            });
            await renderContent({
                content: childrenArr,
                serverlessData,
                parents: parentsCopy,
                pageData,
                pageContains,
                pageHeadings,
                navigations,
                headingsIndex,
                depth: depth + 1
            }, _output);
        }
        _output.html += renderEnd;
    }
    /* Output */
    return _output.html;
};
exports.renderContent = renderContent;
/**
 * Output single post or page
 *
 * @param {RenderItemArgs} args
 * @return {Promise<RenderItemReturn|null>}
 */
const renderItem = async (args) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return null;
    }
    const { item, contentType = 'page', serverlessData } = args;
    /* Item must be object */
    if (!(0, object_js_1.isObjectStrict)(item)) {
        return null;
    }
    /* Taxonomy not page */
    if (contentType === 'taxonomy' && item.isPage !== true) {
        return null;
    }
    /* Item id required */
    const id = item.id;
    if (!(0, string_js_1.isStringStrict)(id)) {
        return null;
    }
    /* Slug required */
    if (!(0, string_js_1.isStringStrict)(item.slug)) {
        return null;
    }
    /* Serverless render check */
    let serverlessRender = false;
    /* Components contained in page  */
    const pageContains = [];
    /* Rich text headings in page */
    const pageHeadings = [];
    /* Reset script and style files */
    scriptStyle_js_1.scripts.deps.clear();
    scriptStyle_js_1.scripts.item.clear();
    scriptStyle_js_1.scripts.meta = {};
    scriptStyle_js_1.styles.deps.clear();
    scriptStyle_js_1.styles.item.clear();
    /* Start action */
    const renderItemStartArgs = {
        id,
        pageData: { ...item },
        contentType,
        pageContains: [],
        pageHeadings: [],
        serverlessData
    };
    await (0, action_js_1.doActions)('renderItemStart', renderItemStartArgs, true);
    /* Meta */
    const title = item.title;
    const meta = {
        title: '',
        description: '',
        url: '',
        image: '',
        canonical: '',
        prev: '',
        next: '',
        noIndex: false,
        isIndex: false,
        ...item.meta
    };
    if ((0, string_js_1.isStringStrict)(item.metaTitle)) {
        meta.title = item.metaTitle;
    }
    if ((0, string_js_1.isStringStrict)(item.metaDescription)) {
        meta.description = item.metaDescription;
    }
    if ((0, string_js_1.isStringStrict)(item.metaImage?.url)) {
        meta.image = item.metaImage.url;
    }
    if (!(0, string_js_1.isStringStrict)(meta.title) && (0, string_js_1.isStringStrict)(title)) {
        meta.title = title;
    }
    /* Term taxonomy */
    const taxonomy = contentType === 'term' ? item.taxonomy : contentType === 'taxonomy' ? item : {};
    /* Permalink */
    const slugArgs = {
        id,
        contentType,
        slug: item.slug,
        returnParents: true,
        pageData: item,
        page: 0
    };
    const s = (0, link_js_1.getSlug)(slugArgs, true);
    const slug = s.slug;
    const slugIsHtml = slug.includes('.html');
    const permalink = (0, link_js_1.getPermalink)(slug, !slugIsHtml);
    const parents = s.parents;
    item.basePermalink = permalink;
    meta.url = permalink;
    meta.canonical = permalink;
    /* Format and add to slug store */
    let formattedSlug = slug !== 'index' && slug !== '' ? `/${slug}/` : '/';
    if (slugIsHtml) {
        formattedSlug = slug;
    }
    (0, store_js_1.setStoreItem)('slugs', { contentType, id }, slugIsHtml ? `/${slug}` : formattedSlug);
    /* Check if index */
    const index = item.slug === 'index';
    meta.isIndex = index;
    /* Navigations */
    let navigations = {};
    if ((0, function_js_1.isFunction)(renderNavigations)) {
        let currentType = contentType;
        if ((0, object_js_1.isObjectStrict)(taxonomy) && (0, array_js_1.isArrayStrict)(taxonomy.contentTypes)) {
            currentType = taxonomy.contentTypes;
        }
        navigations = await renderNavigations({
            navigations: (0, store_js_1.getStoreItem)('navigations'),
            items: (0, store_js_1.getStoreItem)('navigationItems'),
            currentLink: permalink,
            currentType,
            title,
            parents
        });
    }
    /* Serverless data */
    let itemServerlessData;
    if ((0, object_js_1.isObjectStrict)(serverlessData)) {
        const serverlessPath = serverlessData.path;
        const isServerless = serverlessPath === formattedSlug && serverlessData.query != null;
        if (!isServerless) { // Avoid re-rendering non dynamic pages
            return {
                serverlessRender: false
            };
        }
        itemServerlessData = serverlessData;
    }
    /* Page data (props) for layout and actions */
    const pageData = { ...item };
    pageData.id = id;
    pageData.parents = parents;
    pageData.content = undefined;
    /* Content loop */
    let contentOutput = '';
    const contentData = item.content;
    if ((0, string_js_1.isStringStrict)(contentData)) {
        contentOutput = contentData;
    }
    if ((0, array_js_1.isArrayStrict)(contentData)) {
        contentOutput = await renderContent({
            content: contentData,
            serverlessData: itemServerlessData,
            parents: [],
            pageData,
            pageContains,
            pageHeadings,
            navigations
        });
    }
    contentOutput = await (0, shortcode_js_1.doShortcodes)(contentOutput);
    /* Pagination variables for meta object */
    const pag = pageData.pagination;
    if ((0, object_js_1.isObjectStrict)(pag)) {
        const { current = 0, total = 1, currentFilters, prevFilters, nextFilters } = pag;
        meta.canonicalParams =
            `${current > 1 ? `?page=${current}` : ''}${(0, string_js_1.isString)(currentFilters) ? currentFilters : ''}`;
        if ((0, string_js_1.isStringStrict)(pag.title)) {
            meta.paginationTitle = pag.title;
        }
        if ((0, number_js_1.isNumber)(pag.prev) && pag.prev >= 1) {
            slugArgs.page = pag.prev;
            const p = (0, link_js_1.getSlug)(slugArgs, true);
            meta.prev = `${(0, link_js_1.getPermalink)(p.slug, pag.prev === 1)}${(0, string_js_1.isString)(prevFilters) ? prevFilters : ''}`;
        }
        if ((0, number_js_1.isNumber)(pag.next) && pag.next > 1 && pag.next < total) {
            slugArgs.page = pag.next;
            const n = (0, link_js_1.getSlug)(slugArgs, true);
            meta.next = `${(0, link_js_1.getPermalink)(n.slug, false)}${(0, string_js_1.isString)(nextFilters) ? nextFilters : ''}`;
        }
        serverlessRender = true;
    }
    /* Output */
    let layoutOutput = '';
    if ((0, function_js_1.isFunction)(renderLayout)) {
        const layoutArgs = {
            id,
            meta,
            navigations,
            contentType,
            content: contentOutput,
            slug: formattedSlug,
            pageContains,
            pageHeadings,
            pageData,
            serverlessData
        };
        layoutOutput = await renderLayout(layoutArgs);
    }
    const renderItemFilterArgs = {
        id,
        contentType,
        slug: formattedSlug,
        output: layoutOutput,
        pageData,
        pageContains,
        pageHeadings,
        serverlessData
    };
    layoutOutput = await (0, filter_js_1.applyFilters)('renderItem', layoutOutput, renderItemFilterArgs, true);
    /* End action */
    const renderItemEndArgs = {
        id,
        contentType,
        slug: formattedSlug,
        output: layoutOutput,
        pageData,
        pageContains,
        pageHeadings,
        serverlessData
    };
    await (0, action_js_1.doActions)('renderItemEnd', renderItemEndArgs, true);
    /* Output */
    return {
        serverlessRender,
        pageData,
        data: {
            slug: formattedSlug,
            output: layoutOutput
        }
    };
};
exports.renderItem = renderItem;
/**
 * Loop through all content types to output pages and posts
 *
 * @param {RenderArgs} args
 * @return {Promise<RenderReturn[]|RenderReturn>}
 */
const render = async (args) => {
    /* Props must be object */
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return [];
    }
    const { allData, serverlessData, previewData } = args;
    /* Serverless */
    const isServerless = serverlessData != null;
    const isPreview = previewData != null;
    /* Reset script and style directories */
    scriptStyle_js_1.scripts.deps.clear();
    scriptStyle_js_1.scripts.item.clear();
    scriptStyle_js_1.scripts.meta = {};
    scriptStyle_js_1.styles.deps.clear();
    scriptStyle_js_1.styles.item.clear();
    /* Data */
    if (!(0, object_js_1.isObjectStrict)(allData)) {
        return [];
    }
    const { redirect, content } = allData;
    /* Start action */
    await (0, action_js_1.doActions)('renderStart', args, true);
    /* Content data */
    const data = [];
    /* Store data */
    await (0, store_js_1.setStoreData)(allData, isServerless);
    /* Redirects data */
    (0, redirects_js_1.setRedirects)(redirect);
    /* Empty serverless reload */
    serverless_js_1.serverlessRoutes.reload = [];
    /* Loop through all content types */
    for (const [contentType, contentItems] of Object.entries(content)) {
        if (!(0, array_js_1.isArrayStrict)(contentItems)) {
            continue;
        }
        for (const contentItem of contentItems) {
            const item = await renderItem({
                item: contentItem,
                contentType,
                serverlessData
            });
            if (item == null) {
                continue;
            }
            const { serverlessRender = false, data: itemData } = item;
            if (itemData == null) {
                continue;
            }
            data.push(itemData);
            if (serverlessRender && !isServerless) {
                serverless_js_1.serverlessRoutes.reload.push({
                    path: itemData.slug.replace(/^\/|\/$/gm, '')
                });
            }
        }
    }
    /* Output */
    const [outputItem] = data;
    const output = (isServerless || isPreview) && outputItem != null ? outputItem : data;
    await (0, action_js_1.doActions)('renderEnd', { ...args, data: output }, true);
    return output;
};
exports.render = render;
