"use strict";
/**
 * Serverless - Ajax
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ajax = void 0;
const ResponseError_js_1 = require("../../utils/ResponseError/ResponseError.js");
const filter_js_1 = require("../../utils/filter/filter.js");
const object_js_1 = require("../../utils/object/object.js");
const string_js_1 = require("../../utils/string/string.js");
const number_js_1 = require("../../utils/number/number.js");
const function_js_1 = require("../../utils/function/function.js");
const print_js_1 = require("../../utils/print/print.js");
const config_js_1 = require("../../config/config.js");
const serverless_js_1 = require("../serverless.js");
/**
 * Handle ajax requests by processing data and calling serverless actions
 *
 * @param {ServerlessContext} context
 * @param {ServerlessSetup} setupServerless
 * @return {Promise<Response>} Response
 */
const Ajax = async (context, setupServerless) => {
    try {
        /* Setup */
        setupServerless(context);
        /* Get form data */
        const request = context.request;
        const data = await request.json();
        /* Data must be object */
        if (!(0, object_js_1.isObjectStrict)(data)) {
            throw new Error('Data not an object');
        }
        /* Honeypot check */
        const honeypotName = `${config_js_1.config.namespace}_asi`;
        if ((0, object_js_1.isObjectStrict)(data.inputs[honeypotName])) {
            const honeypotValue = data.inputs[honeypotName].value;
            if ((0, string_js_1.isStringStrict)(honeypotValue)) {
                const options = {
                    status: 200,
                    headers: {
                        'Content-Type': 'application/json'
                    }
                };
                return new Response(JSON.stringify({ success: 'Form successully sent' }), options);
            }
            data.inputs[honeypotName].exclude = true;
        }
        /* Action required */
        const action = data.action;
        if (!(0, string_js_1.isStringStrict)(action)) {
            throw new Error('No action');
        }
        /* Call functions by action */
        let res = null;
        const ajaxFn = serverless_js_1.serverlessActions[action];
        if ((0, function_js_1.isFunction)(ajaxFn)) {
            res = await ajaxFn(data, context);
        }
        const ajaxResultFilterArgs = {
            data,
            context
        };
        res = await (0, filter_js_1.applyFilters)('ajaxResult', res, ajaxResultFilterArgs, true);
        /* Result error */
        if (res == null) {
            throw new Error('No result');
        }
        if (res.error != null) {
            throw new ResponseError_js_1.ResponseError(res.error.message, res.error.response);
        }
        /* Result success */
        const options = {
            status: 200,
            headers: {
                'Content-Type': 'application/json'
            }
        };
        let message = '';
        if (res.success != null) {
            const { message: successMessage, headers } = res.success;
            if ((0, string_js_1.isStringStrict)(successMessage)) {
                message = successMessage;
            }
            if ((0, object_js_1.isObjectStrict)(headers)) {
                options.headers = { ...options.headers, ...headers };
            }
        }
        return new Response(JSON.stringify({ success: message }), options);
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error with ajax function', error);
        let statusCode = 500;
        let message = '';
        if (error instanceof ResponseError_js_1.ResponseError) {
            if ((0, number_js_1.isNumber)(error.response?.status)) {
                statusCode = error.response.status;
            }
            if ((0, string_js_1.isStringStrict)(error.message)) {
                message = error.message;
            }
        }
        const options = {
            status: statusCode,
            headers: {
                'Content-Type': 'application/json'
            }
        };
        return new Response(JSON.stringify({ error: message }), options);
    }
};
exports.Ajax = Ajax;
