"use strict";
/**
 * Serverless - Reload
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Reload = void 0;
const string_js_1 = require("../../utils/string/string.js");
const object_js_1 = require("../../utils/object/object.js");
const function_js_1 = require("../../utils/function/function.js");
const print_js_1 = require("../../utils/print/print.js");
const render_js_1 = require("../../render/render.js");
/**
 * Error html output
 *
 * @private
 * @param {ServerlessContext} context
 * @param {ServerlessSetup} setupServerless
 * @param {number} status
 * @return {Promise<string>} html
 */
const getErrorHtml = async (context, setupServerless, status) => {
    try {
        setupServerless(context);
        if ((0, function_js_1.isFunction)(render_js_1.renderHttpError)) {
            return await (0, render_js_1.renderHttpError)({ code: status });
        }
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error rendering http error', error);
    }
    return '';
};
/**
 * Output paginated and/or filtered page on browser reload
 *
 * @param {ServerlessContext} context
 * @param {ServerlessSetup} setupServerless
 * @param {function} getData - getAllContentfulData | getAllWordPressData
 * @return {Promise<Response>} Response
 */
const Reload = async (context, setupServerless, getData) => {
    try {
        /* Query */
        const { request, next } = context;
        const { searchParams, pathname } = new URL(request.url);
        const page = searchParams.get('page');
        const filters = searchParams.get('filters');
        const path = pathname;
        const query = {};
        let noPage = false;
        let noFilters = false;
        if ((0, string_js_1.isStringStrict)(page)) {
            query.page = page;
        }
        else {
            noPage = true;
        }
        if ((0, string_js_1.isStringStrict)(filters)) {
            query.filters = filters;
        }
        else {
            noFilters = true;
        }
        /* No query move on to default page */
        if (noPage && noFilters) {
            return next(); // eslint-disable-line @typescript-eslint/return-await
        }
        /* Setup */
        setupServerless(context);
        /* Data params */
        const serverlessData = { query, path };
        /* Output */
        const data = await (0, render_js_1.render)({
            serverlessData,
            allData: await getData({
                serverlessData
            })
        });
        let html = '';
        let status = 200;
        if ((0, object_js_1.isObjectStrict)(data)) {
            html = data.output;
        }
        const isEmpty = html === '';
        if (isEmpty) {
            status = 404;
        }
        if (isEmpty && (0, function_js_1.isFunction)(render_js_1.renderHttpError)) {
            html = await (0, render_js_1.renderHttpError)({ code: status });
        }
        return new Response(html, {
            status,
            headers: {
                'Content-Type': 'text/html;charset=UTF-8'
            }
        });
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error with reload function', error);
        const status = 500;
        const html = await getErrorHtml(context, setupServerless, status);
        return new Response(html, {
            status,
            headers: {
                'Content-Type': 'text/html;charset=UTF-8'
            }
        });
    }
};
exports.Reload = Reload;
