"use strict";
/**
 * Serverless - Send Form
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SendForm = void 0;
const config_js_1 = require("../../config/config.js");
const escape_js_1 = require("../../utils/escape/escape.js");
const array_js_1 = require("../../utils/array/array.js");
const string_js_1 = require("../../utils/string/string.js");
const object_js_1 = require("../../utils/object/object.js");
const boolean_js_1 = require("../../utils/boolean/boolean.js");
const objectUtils_js_1 = require("../../utils/object/objectUtils.js");
const link_js_1 = require("../../utils/link/link.js");
const store_js_1 = require("../../store/store.js");
const serverless_js_1 = require("../serverless.js");
/**
 * Recurse through data to output plain and html email body
 *
 * @private
 * @param {object} data
 * @param {Object<string, string>} output
 * @param {string} output.html
 * @param {string} output.plain
 * @param {number} depth
 * @return {void}
 */
const recurseEmailHtml = (data, output, depth = 1) => {
    if (!(0, object_js_1.isObject)(data)) {
        return;
    }
    const isArr = (0, array_js_1.isArray)(data);
    (0, objectUtils_js_1.getObjectKeys)(data).forEach((label) => {
        const value = data[label];
        const l = label.toString();
        const h = depth + 1;
        if (depth === 1) {
            output.html += `
        <tr>
          <td style="padding: 16px 0; border-bottom: 2px solid #ccc;">
      `;
        }
        if (label !== '' && !isArr) {
            output.html += `
        <h${h} style="font-family: sans-serif; color: #222; margin: 16px 0; line-height: 1.3em">
          ${l}
        </h${h}>
      `;
            output.plain += `${l}\n`;
        }
        recurseEmailHtml(value, output, depth + 1);
        if ((0, string_js_1.isString)(value)) {
            output.html += `
        <p style="font-family: sans-serif; color: #222; margin: 16px 0; line-height: 1.5em;">
          ${value}
        </p>
      `;
            output.plain += value
                .replace(/<br\s*\/?>/gi, '\n')
                .replace(/(<([^>]+)>)/ig, '') + '\n';
        }
        if (depth === 1) {
            output.html += `
          </td>
        </tr>
      `;
            output.plain += '\n';
        }
    });
};
/**
 * Generate email from form fields and send with Smtp2go
 *
 * @type {ServerlessAction}
 */
const SendForm = async (args) => {
    /* Api key */
    const apiKey = serverless_js_1.serverlessApiKeys.smtp2go;
    if (!(0, string_js_1.isStringStrict)(apiKey)) {
        return {
            error: {
                message: 'No api key'
            }
        };
    }
    /* Args */
    const { id, inputs } = args;
    /* Id required */
    if (!(0, string_js_1.isStringStrict)(id)) {
        return {
            error: {
                message: 'No id'
            }
        };
    }
    /* Inputs required */
    if (!(0, object_js_1.isObjectStrict)(inputs) || Object.keys(inputs).length === 0) {
        return {
            error: {
                message: 'No inputs'
            }
        };
    }
    /* Meta information - to email and subject */
    const formMetaData = await (0, store_js_1.fetchStoreItem)('formMeta');
    if (!(0, object_js_1.isObjectStrict)(formMetaData)) {
        return {
            error: {
                message: 'No meta'
            }
        };
    }
    const meta = formMetaData?.[id];
    if (!(0, object_js_1.isObjectStrict)(meta)) {
        return {
            error: {
                message: 'No meta object'
            }
        };
    }
    /* To email */
    const toEmail = meta.toEmail;
    if (!(0, string_js_1.isStringStrict)(toEmail)) {
        return {
            error: {
                message: 'No to email'
            }
        };
    }
    const toEmails = toEmail.split(',');
    /* Sender email */
    const senderEmail = meta.senderEmail;
    if (!(0, string_js_1.isStringStrict)(senderEmail)) {
        return {
            error: {
                message: 'No sender email'
            }
        };
    }
    /* Subject */
    let subject = (0, string_js_1.isStringStrict)(meta.subject) ? meta.subject : '';
    /* Reply to email */
    let replyToEmail = '';
    /* Email content */
    const header = `${config_js_1.config.title} contact form submission`;
    const footer = `This email was sent from a contact form on ${config_js_1.config.title} (${(0, link_js_1.getPermalink)()})`;
    const outputData = {};
    const output = {
        html: '',
        plain: ''
    };
    for (const [name, input] of Object.entries(inputs)) {
        /* Skip if exclude true */
        const inputExclude = input.exclude;
        const exclude = (0, boolean_js_1.isBoolean)(inputExclude) ? inputExclude : false;
        if (exclude) {
            continue;
        }
        /* Variables */
        const inputType = input.type;
        const inputLabel = input.label.trim();
        const inputValue = input.value;
        /* Escape value */
        let inputValueStr = '';
        if ((0, array_js_1.isArray)(inputValue)) {
            inputValueStr = inputValue.map(v => (0, escape_js_1.escape)(v.trim() + '')).join('<br>');
        }
        if ((0, string_js_1.isString)(inputValue)) {
            inputValueStr = (0, escape_js_1.escape)(inputValue.trim() + '');
        }
        /* Subject */
        if (name === 'subject') {
            subject = inputValueStr !== '' ? `${(0, string_js_1.isStringStrict)(subject) ? `${subject} - ` : ''}${inputValueStr}` : subject;
            continue;
        }
        /* Reply to email */
        if (inputType === 'email' && inputValueStr !== '') {
            replyToEmail = inputValueStr;
            inputValueStr = `<a href="mailto:${inputValueStr}">${inputValueStr}</a>`;
        }
        /* Output value */
        const outputValue = inputValueStr === '' ? '--' : inputValueStr;
        /* Legend */
        let hasLegend = false;
        let legend = '';
        if ((0, string_js_1.isStringStrict)(input.legend)) {
            hasLegend = true;
            legend = input.legend;
        }
        if (hasLegend) {
            const legendData = outputData[legend];
            if (legendData == null) {
                outputData[legend] = {};
            }
            // @ts-expect-error
            const inputData = outputData[legend][inputLabel];
            if (inputData == null) {
                // @ts-expect-error
                outputData[legend][inputLabel] = [];
            }
            // @ts-expect-error
            outputData[legend][inputLabel].push(outputValue);
        }
        /* Label */
        if (!hasLegend) {
            const inputData = outputData[inputLabel];
            if (inputData == null) {
                outputData[inputLabel] = [];
            }
            // @ts-expect-error
            outputData[inputLabel].push(outputValue);
        }
    }
    recurseEmailHtml(outputData, output);
    const outputHtml = `
    <table width="100%" cellpadding="0" cellspacing="0" border="0">
      <tr>
        <td align="center" width="100%" style="padding: 0 16px 16px 16px;">
          <table align="center" cellpadding="0" cellspacing="0" border="0" style="margin-right: auto; margin-left: auto; border-spacing: 0; max-width: 37.5em;">
            <tr>
              <td style="padding: 32px 0 0 0;">
                <h1 style="font-family: sans-serif; color: #222; margin: 0; line-height: 1.3em;">
                  ${header}
                </h1>
              </td>
            </tr>
            <tr>
              <td>
                <table border="0" cellpadding="0" cellspacing="0" width="100%" style="border-collapse: collapse;">
                  ${output.html}
                  <tr>
                    <td style="padding: 32px 0;">
                      <p style="font-family: sans-serif; color: #222; margin: 0; line-height: 1.5em;">
                        ${footer}
                      </p>
                    </td>
                  </tr>
                </table>
              </td>
            </tr>
          </table>
        </td>
      </tr>
    </table>
  `;
    const outputPlain = `${header}\n\n${output.plain}${footer}`;
    /* Subjext fallback */
    if (subject === '') {
        subject = `${config_js_1.config.title} Contact Form`;
    }
    /* Smtp2go request */
    const body = {
        api_key: apiKey,
        to: toEmails,
        sender: senderEmail,
        subject,
        text_body: outputPlain,
        html_body: outputHtml
    };
    if (replyToEmail !== '') {
        body.custom_headers = [
            {
                header: 'Reply-To',
                value: `<${replyToEmail}>`
            }
        ];
    }
    const res = await fetch('https://api.smtp2go.com/v3/email/send', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(body)
    });
    /* Success */
    if (res.ok) {
        return {
            success: {
                message: 'Form successully sent'
            }
        };
    }
    else {
        return {
            error: {
                message: 'Error sending email',
                response: res
            }
        };
    }
};
exports.SendForm = SendForm;
