"use strict";
/**
 * Serverless - Files
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServerlessFiles = void 0;
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const path_js_1 = require("../utils/path/path.js");
const object_js_1 = require("../utils/object/object.js");
const array_js_1 = require("../utils/array/array.js");
const string_js_1 = require("../utils/string/string.js");
const dataSource_js_1 = require("../utils/dataSource/dataSource.js");
const print_js_1 = require("../utils/print/print.js");
const config_js_1 = require("../config/config.js");
const serverless_js_1 = require("./serverless.js");
/**
 * Minify template strings
 *
 * @private
 * @param {string} str
 * @return {string}
 */
const minify = (str) => {
    return str
        .replace(/\s+/g, ' ')
        .replace(/;\s*}/g, ';}')
        .trim();
};
/**
 * Create files for serverless functions
 *
 * @param {ServerlessFilesArgs} [args]
 * @return {Promise<void>}
 */
const createServerlessFiles = async (args) => {
    /* Defaults and args */
    let defaultDataExport = 'getAllContentfulData';
    let defaultDataFile = '@alanizcreative/static-site-formation/contentful/contentfulData.js';
    if (dataSource_js_1.dataSource.isWordPress()) {
        defaultDataExport = 'getAllWordPressData';
        defaultDataFile = '@alanizcreative/static-site-formation/wordpress/wordpressData.js';
    }
    const defaults = {
        dataExport: defaultDataExport,
        dataExportFile: defaultDataFile,
        setupExport: 'setupServerless',
        setupExportFile: 'lib/setup/setupServerless.js',
        previewExportFile: '@alanizcreative/static-site-formation/serverless/Preview/Preview.js',
        reloadExportFile: '@alanizcreative/static-site-formation/serverless/Reload/Reload.js',
        ajaxExportFile: '@alanizcreative/static-site-formation/serverless/Ajax/Ajax.js',
        ajaxFile: 'ajax/index.js',
        previewFile: '_middleware.js',
        reloadFile: '_middleware.js'
    };
    const { dataExport, dataExportFile, setupExport, setupExportFile, previewExportFile, reloadExportFile, ajaxExportFile, ajaxFile, previewFile, reloadFile } = Object.assign(defaults, (0, object_js_1.isObjectStrict)(args) ? args : {});
    /* Serverless folder required */
    await (0, promises_1.mkdir)((0, node_path_1.resolve)(serverless_js_1.serverlessDir), { recursive: true });
    /* Ajax file */
    if ((0, string_js_1.isStringStrict)(ajaxFile)) {
        const pathDepth = (0, path_js_1.getPathDepth)(`${serverless_js_1.serverlessDir}/${ajaxFile}`);
        const content = `
      import { Ajax } from '${ajaxExportFile.startsWith('@') ? '' : pathDepth}${ajaxExportFile}';
      import { ${setupExport} } from '${pathDepth}${setupExportFile}';
      const render = async (context) => {
        return await Ajax(context, ${setupExport});
      };
      export const onRequestPost = [render];
    `;
        const path = (0, node_path_1.resolve)(serverless_js_1.serverlessDir, ajaxFile);
        const dir = (0, node_path_1.dirname)(path);
        await (0, promises_1.mkdir)((0, node_path_1.resolve)(serverless_js_1.serverlessDir, dir), { recursive: true });
        await (0, promises_1.writeFile)(path, minify(content));
        (0, print_js_1.print)('[SSF] Successfully wrote', path, 'success');
    }
    /* Preview file */
    if (config_js_1.config.env.dev && (0, string_js_1.isStringStrict)(previewFile)) {
        const pathDepth = (0, path_js_1.getPathDepth)(`${serverless_js_1.serverlessDir}/${previewFile}`);
        const content = `
      import { Preview } from '${previewExportFile.startsWith('@') ? '' : pathDepth}${previewExportFile}';
      import { ${dataExport} } from '${dataExportFile.startsWith('@') ? '' : pathDepth}${dataExportFile}';
      import { ${setupExport} } from '${pathDepth}${setupExportFile}';
      const render = async (context) => {
        return await Preview(context, ${setupExport}, ${dataExport});
      };
      export const onRequestGet = [render];
    `;
        const path = (0, node_path_1.resolve)(serverless_js_1.serverlessDir, previewFile);
        const dir = (0, node_path_1.dirname)(path);
        await (0, promises_1.mkdir)((0, node_path_1.resolve)(serverless_js_1.serverlessDir, dir), { recursive: true });
        await (0, promises_1.writeFile)(path, minify(content));
        (0, print_js_1.print)('[SSF] Successfully wrote', path, 'success');
    }
    /* Routes */
    for (const [type, routes] of Object.entries(serverless_js_1.serverlessRoutes)) {
        if (!(0, array_js_1.isArrayStrict)(routes)) {
            continue;
        }
        for (const route of routes) {
            let { path = '', content = '' } = route;
            if (!(0, string_js_1.isStringStrict)(path)) {
                continue;
            }
            if (type === 'reload' && (0, string_js_1.isStringStrict)(reloadFile)) {
                path = `${path}/${reloadFile}`;
                const pathDepth = (0, path_js_1.getPathDepth)(`${serverless_js_1.serverlessDir}/${path}`);
                content = `
          import { Reload } from '${reloadExportFile.startsWith('@') ? '' : pathDepth}${reloadExportFile}';
          import { ${dataExport} } from '${dataExportFile.startsWith('@') ? '' : pathDepth}${dataExportFile}';
          import { ${setupExport} } from '${pathDepth}${setupExportFile}';
          const render = async (context) => {
            return await Reload(context, ${setupExport}, ${dataExport});
          };
          export const onRequestGet = [render];
        `;
            }
            if ((0, string_js_1.isStringStrict)(content)) {
                path = (0, node_path_1.resolve)(serverless_js_1.serverlessDir, path);
                const dir = (0, node_path_1.dirname)(path);
                await (0, promises_1.mkdir)((0, node_path_1.resolve)(serverless_js_1.serverlessDir, dir), { recursive: true });
                await (0, promises_1.writeFile)(path, content);
                (0, print_js_1.print)('[SSF] Successfully wrote', path, 'success');
            }
        }
    }
};
exports.createServerlessFiles = createServerlessFiles;
