"use strict";
/**
 * Store
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getStoreItem = exports.setStoreItem = exports.fetchStoreItem = exports.setStoreData = exports.setStore = exports.storeDir = exports.store = void 0;
const contentType_js_1 = require("../utils/contentType/contentType.js");
const object_js_1 = require("../utils/object/object.js");
const string_js_1 = require("../utils/string/string.js");
const array_js_1 = require("../utils/array/array.js");
const json_js_1 = require("../utils/json/json.js");
const config_js_1 = require("../config/config.js");
/**
 * Directory to write store files to
 *
 * @type {string}
 */
let storeDir = 'lib/store';
exports.storeDir = storeDir;
/**
 * Default store object
 *
 * @type {Store}
 */
const defaultStore = {
    slugs: {},
    parents: {},
    navigations: [],
    navigationItems: [],
    formMeta: {},
    archiveMeta: {},
    imageMeta: {}
};
/**
 * Store options
 *
 * @type {Store}
 */
let store = {
    ...defaultStore
};
exports.store = store;
/**
 * Update default store with user options
 *
 * @param {Store} args
 * @param {string} [dir]
 * @return {Store}
 */
const setStore = (args, dir = 'lib/store') => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return store;
    }
    exports.store = store = { ...defaultStore, ...args };
    if ((0, string_js_1.isStringStrict)(dir)) {
        exports.storeDir = storeDir = dir;
    }
    return store;
};
exports.setStore = setStore;
/**
 * Update individual store property
 *
 * @param {string} prop
 * @param {object} value
 * @param {string} [subProp]
 * @return {boolean}
 */
const setStoreItem = (prop, value, subProp) => {
    if (!(0, string_js_1.isStringStrict)(prop) || !(0, object_js_1.isObject)(value) || store[prop] == null) {
        return false;
    }
    if ((0, string_js_1.isStringStrict)(subProp)) {
        store[prop][subProp] = value;
    }
    else {
        store[prop] = value;
    }
    return true;
};
exports.setStoreItem = setStoreItem;
/**
 * Set serverless or build time data (navigations, archive meta, parents)
 *
 * @param {RenderAllData} allData
 * @param {boolean} [isServerless]
 * @return {Promise<boolean>}
 */
const setStoreData = async (allData, isServerless = false) => {
    /* Serverless all data */
    if (isServerless) {
        for (const key of Object.keys(store)) {
            if (key === 'slugs') { // Skip slugs as set in data fetch
                continue;
            }
            const data = await (0, json_js_1.getJsonFile)(key);
            if (data != null) {
                store[key] = data;
            }
        }
        return true;
    }
    /* Data must be object */
    if (!(0, object_js_1.isObjectStrict)(allData)) {
        return false;
    }
    const { navigation, navigationItem, content } = allData;
    /* Build time navigation data */
    store.navigations = (0, array_js_1.isArrayStrict)(navigation) ? navigation : [];
    store.navigationItems = (0, array_js_1.isArrayStrict)(navigationItem) ? navigationItem : [];
    /* Build time archive meta and parent data */
    const data = {
        ...allData,
        ...content
    };
    config_js_1.config.hierarchicalTypes.forEach(type => {
        const items = data?.[type];
        if (!(0, array_js_1.isArrayStrict)(items)) {
            return;
        }
        items.forEach(item => {
            if (!(0, object_js_1.isObjectStrict)(item)) {
                return;
            }
            /* Props */
            const { id, parent, archive, slug, title } = item;
            /* Id required */
            if (!(0, string_js_1.isStringStrict)(id)) {
                return;
            }
            /* Archive */
            const archiveType = (0, contentType_js_1.normalizeContentType)(archive);
            if ((0, string_js_1.isStringStrict)(archiveType)) {
                const archiveObj = (0, object_js_1.isObjectStrict)(store.archiveMeta[archiveType]) ? store.archiveMeta[archiveType] : {};
                store.archiveMeta[archiveType] = {
                    id,
                    slug,
                    title,
                    contentType: type,
                    ...archiveObj
                };
            }
            /* Parent */
            if ((0, object_js_1.isObjectStrict)(parent)) {
                const parentSlug = parent.slug;
                const parentTitle = parent.title;
                const parentId = parent.id;
                if ((0, string_js_1.isStringStrict)(parentSlug) && (0, string_js_1.isStringStrict)(parentTitle) && (0, string_js_1.isStringStrict)(parentId)) {
                    if (store.parents[type] == null) {
                        store.parents[type] = {};
                    }
                    store.parents[type][id] = {
                        id: parentId,
                        slug: parentSlug,
                        title: parentTitle
                    };
                }
            }
        });
    });
    return true;
};
exports.setStoreData = setStoreData;
/**
 * Get individual store property
 *
 * @param {string} prop
 * @return {object}
 */
const getStoreItem = (prop) => {
    return store[prop];
};
exports.getStoreItem = getStoreItem;
/**
 * Set and get serverless store property
 *
 * @param {string} prop
 * @return {object}
 */
const fetchStoreItem = async (prop) => {
    const data = await (0, json_js_1.getJsonFile)(prop);
    if (data != null) {
        store[prop] = data;
    }
    return store[prop];
};
exports.fetchStoreItem = fetchStoreItem;
