"use strict";
/**
 * Text - Rich Text
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.RichText = void 0;
const link_js_1 = require("../../utils/link/link.js");
const excerpt_js_1 = require("../../utils/excerpt/excerpt.js");
const filter_js_1 = require("../../utils/filter/filter.js");
const string_js_1 = require("../../utils/string/string.js");
const array_js_1 = require("../../utils/array/array.js");
const object_js_1 = require("../../utils/object/object.js");
const heading_js_1 = require("../../utils/heading/heading.js");
/**
 * Check if string contains shortcode
 *
 * @private
 * @param {string} tag
 * @param {string} content
 * @return {boolean}
 */
const containsShortcode = (tag = '', content = '') => {
    if (tag === 'p' && content.charAt(0) === '[' && content.charAt(content.length - 1) === ']') {
        return true;
    }
    return false;
};
/**
 * Check if data rich attribute should be added
 *
 * @private
 * @param {boolean|string[]} dataAttr
 * @param {string} tag
 * @return {boolean}
 */
const addDataAttr = (dataAttr = true, tag = '') => {
    if ((0, array_js_1.isArrayStrict)(dataAttr)) {
        return dataAttr.includes(tag);
    }
    return dataAttr;
};
/**
 * Recursively output content
 *
 * @private
 * @param {RichTextContentProps} args
 * @return {string}
 */
const getContent = (args) => {
    const { content = [], props, dataAttr = false } = args;
    let { _output = '' } = args;
    for (let item of content) {
        item = (0, filter_js_1.applyFilters)('richTextContentItem', item, props);
        item = (0, object_js_1.isObjectStrict)(item) ? item : {};
        const { link = '', attr = '', internalLink, content: c } = item;
        let { tag = '' } = item;
        let cc = c;
        /* Nested content */
        if ((0, array_js_1.isArrayStrict)(c)) {
            cc = getContent({
                content: c,
                props,
                dataAttr
            });
        }
        /* Attributes */
        const attrs = [];
        if (addDataAttr(dataAttr, tag)) {
            attrs.push(`data-rich="${tag}"`);
        }
        if ((0, string_js_1.isStringStrict)(attr)) {
            attrs.push(attr);
        }
        /* Link */
        if (tag === 'a') {
            let anchorLink = link;
            if (internalLink != null) {
                anchorLink = (0, link_js_1.getLink)(internalLink);
            }
            if ((0, string_js_1.isStringStrict)(anchorLink)) {
                attrs.push(`href="${anchorLink}"`);
            }
        }
        /* Filter output */
        let outputStr = '';
        if ((0, string_js_1.isString)(cc)) {
            const richTextContentFilterArgs = {
                args: item,
                props
            };
            const ccc = (0, filter_js_1.applyFilters)('richTextContent', cc, richTextContentFilterArgs);
            if ((0, string_js_1.isString)(ccc)) {
                cc = ccc;
            }
            outputStr += cc;
        }
        if (containsShortcode(tag, outputStr)) {
            tag = '';
        }
        /* Output */
        let opening = '';
        let closing = '';
        let inner = '';
        if ((0, string_js_1.isStringStrict)(tag) && outputStr.trim() !== '') {
            opening = `<${tag}${(attrs.length > 0) ? ` ${attrs.join(' ')}` : ''}>`;
            closing = `</${tag}>`;
            inner = outputStr;
            outputStr = `${opening}${outputStr}${closing}`;
        }
        const richTextContentOutputArgs = {
            args: item,
            props,
            element: {
                opening,
                closing,
                content: inner
            }
        };
        outputStr = (0, filter_js_1.applyFilters)('richTextContentOutput', outputStr, richTextContentOutputArgs);
        _output += outputStr;
    }
    /* Output */
    return _output;
};
/**
 * Output rich text
 *
 * @param {RichTextProps} props
 * @return {string}
 */
const RichText = (props) => {
    /* Props must be object */
    if (!(0, object_js_1.isObjectStrict)(props)) {
        return '';
    }
    props = (0, filter_js_1.applyFilters)('richTextProps', props, { renderType: 'richText' });
    /* Filtered props must be object */
    if (!(0, object_js_1.isObjectStrict)(props)) {
        return '';
    }
    const { args } = props;
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return '';
    }
    const { content = [], classes = '', link = '', internalLink, textStyle = '', headingStyle = '', align = '', style = '', attr = '', dataAttr = true } = args;
    let { tag = '' } = args;
    /* Hr */
    if (tag === 'hr') {
        return '<hr>';
    }
    /* Check if heading */
    const isSectionHeading = (0, heading_js_1.isHeading)(tag);
    /* Classes */
    const classesArr = [];
    if ((0, string_js_1.isStringStrict)(classes)) {
        classesArr.push(classes);
    }
    if ((0, string_js_1.isStringStrict)(textStyle)) {
        classesArr.push(textStyle);
    }
    if ((0, string_js_1.isStringStrict)(headingStyle) && isSectionHeading) {
        classesArr.push(headingStyle);
    }
    if ((0, string_js_1.isStringStrict)(align)) {
        classesArr.push(align);
    }
    /* Simpler props */
    const filterProps = {
        ...props
    };
    filterProps.args.content = undefined;
    /* Generate output */
    let output = '';
    let headingStr = '';
    let headingObj;
    if ((0, string_js_1.isStringStrict)(content)) {
        output = content;
        headingStr = content;
    }
    if ((0, array_js_1.isArrayStrict)(content)) {
        headingObj = content;
        output = getContent({
            content,
            props: filterProps,
            dataAttr
        });
    }
    /* Attributes */
    const attrs = [];
    if (addDataAttr(dataAttr, tag)) {
        attrs.push(`data-rich="${tag}"`);
    }
    if (isSectionHeading) {
        const headingContents = (0, excerpt_js_1.getExcerpt)({
            limit: 10,
            limitExcerpt: true,
            excerpt: headingStr,
            content: headingObj
        });
        const id = headingContents
            .trim()
            .replace(/&hellip;/g, '')
            .replace(/[^\w\s]|_/g, '')
            .replace(/\s+/g, '-')
            .toLowerCase();
        if (headingContents !== '' && id !== '') {
            attrs.push(`id="${id}"`);
            if ((0, array_js_1.isArray)(props.headings)) {
                props.headings.push({
                    id,
                    title: headingContents,
                    type: tag
                });
            }
        }
    }
    if (tag === 'a') {
        let anchorLink = link;
        const inLink = (0, link_js_1.getLink)(internalLink);
        if ((0, string_js_1.isStringStrict)(inLink)) {
            anchorLink = inLink;
        }
        if ((0, string_js_1.isStringStrict)(anchorLink)) {
            attrs.push(`href="${anchorLink}"`);
        }
    }
    if (classesArr.length > 0) {
        attrs.push(`class="${classesArr.join(' ')}"`);
    }
    if ((0, string_js_1.isStringStrict)(style)) {
        attrs.push(`style="${style}"`);
    }
    if ((0, string_js_1.isStringStrict)(attr)) {
        attrs.push(attr);
    }
    /* Output */
    if (containsShortcode(tag, output)) {
        tag = '';
    }
    let opening = '';
    let closing = '';
    let inner = '';
    if ((0, string_js_1.isStringStrict)(tag) && output.trim() !== '') {
        opening = `<${tag}${(attrs.length > 0) ? ` ${attrs.join(' ')}` : ''}>`;
        closing = `</${tag}>`;
        inner = output;
        output = `${opening}${output}${closing}`;
    }
    const richTextOutputArgs = {
        props: filterProps,
        element: {
            opening,
            closing,
            content: inner
        }
    };
    output = (0, filter_js_1.applyFilters)('richTextOutput', output, richTextOutputArgs);
    return output;
};
exports.RichText = RichText;
