"use strict";
/**
 * Utils - Action
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.setActions = exports.resetActions = exports.doActions = exports.removeAction = exports.addAction = exports.actions = void 0;
const set_js_1 = require("../set/set.js");
const string_js_1 = require("../string/string.js");
const object_js_1 = require("../object/object.js");
const function_js_1 = require("../function/function.js");
/**
 * Action callbacks by name
 *
 * @type {ActionMap}
 */
let actions = new Map([
    ['renderStart', new Set()],
    ['renderEnd', new Set()],
    ['renderItemStart', new Set()],
    ['renderItemEnd', new Set()]
]);
exports.actions = actions;
/**
 * Add action to action map
 *
 * @param {string} name
 * @param {function} action
 * @return {boolean}
 */
const addAction = (name, action) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, function_js_1.isFunction)(action)) {
        return false;
    }
    if (!(0, set_js_1.isSet)(actions.get(name))) {
        actions.set(name, new Set());
    }
    const actionSet = actions.get(name);
    if ((0, set_js_1.isSet)(actionSet)) {
        actionSet.add(action);
    }
    return true;
};
exports.addAction = addAction;
/**
 * Remove action from actions map
 *
 * @param {string} name
 * @param {function} action
 * @return {boolean}
 */
const removeAction = (name, action) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, function_js_1.isFunction)(action)) {
        return false;
    }
    const actionSet = actions.get(name);
    if (!(0, set_js_1.isSet)(actionSet)) {
        return false;
    }
    return actionSet.delete(action);
};
exports.removeAction = removeAction;
/**
 * Call asynchronous functions sequentially
 *
 * @private
 * @param {function[]} callbacks
 * @param {*} [args]
 * @return {void}
 */
const doSequentially = async (callbacks, args) => {
    for (const callback of callbacks) {
        await callback(args);
    }
};
/**
 * Run callback functions from actions map
 *
 * @param {string} name
 * @param {*} [args]
 * @param {boolean} [isAsync]
 * @return {*}
 */
const doActions = (name, args, isAsync = false) => {
    const actionSet = actions.get(name);
    if (!(0, set_js_1.isSetStrict)(actionSet)) {
        return undefined;
    }
    const callbacks = [];
    for (const callback of actionSet.values()) {
        if (isAsync) {
            callbacks.push(callback);
        }
        else {
            callback(args);
        }
    }
    if (isAsync) {
        doSequentially(callbacks, args)
            .then(result => result)
            .catch(() => false);
    }
    return undefined;
};
exports.doActions = doActions;
/**
 * Empty actions map
 *
 * @return {void}
 */
const resetActions = () => {
    exports.actions = actions = new Map([
        ['renderStart', new Set()],
        ['renderEnd', new Set()],
        ['renderItemStart', new Set()],
        ['renderItemEnd', new Set()]
    ]);
};
exports.resetActions = resetActions;
/**
 * Fill actions map
 *
 * @param {Actions} args
 * @return {boolean}
 */
const setActions = (args) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return false;
    }
    const newActions = Object.entries(args);
    if (newActions.length === 0) {
        return false;
    }
    resetActions();
    newActions.forEach(([name, action]) => {
        if (action == null) {
            return;
        }
        addAction(name, action);
    });
    return true;
};
exports.setActions = setActions;
