"use strict";
/**
 * Utils - Archive
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getArchiveLabels = exports.getArchiveLink = exports.getTaxonomyInfo = exports.getArchiveInfo = void 0;
const store_js_1 = require("../../store/store.js");
const object_js_1 = require("../object/object.js");
const array_js_1 = require("../array/array.js");
const string_js_1 = require("../string/string.js");
const boolean_js_1 = require("../boolean/boolean.js");
const contentType_js_1 = require("../contentType/contentType.js");
const link_js_1 = require("../link/link.js");
/**
 * Get archive id, slug and title
 *
 * @param {string} contentType
 * @return {ArchiveInfo}
 */
const getArchiveInfo = (contentType) => {
    const value = {
        id: '',
        slug: '',
        title: '',
        contentType: ''
    };
    contentType = (0, contentType_js_1.normalizeContentType)(contentType);
    if (contentType === 'page') {
        return value;
    }
    const info = (0, store_js_1.getStoreItem)('archiveMeta')[contentType];
    if (!(0, object_js_1.isObjectStrict)(info)) {
        return value;
    }
    const { id, slug, title, contentType: type } = info;
    value.id = (0, string_js_1.isStringStrict)(id) ? id : '';
    value.slug = (0, string_js_1.isStringStrict)(slug) ? slug : '';
    value.title = (0, string_js_1.isStringStrict)(title) ? title : '';
    value.contentType = (0, string_js_1.isStringStrict)(type) ? type : '';
    return value;
};
exports.getArchiveInfo = getArchiveInfo;
/**
 * Get taxonomy attributes
 *
 * @param {string} contentType
 * @param {RenderItem} [pageData]
 * @return {ArchiveTaxonomy}
 */
const getTaxonomyInfo = (contentType, pageData) => {
    const value = {
        id: '',
        slug: '',
        title: '',
        contentTypes: [],
        primaryContentType: '',
        usePrimaryContentTypeSlug: true,
        isPage: false
    };
    const taxObj = contentType === 'taxonomy' ? pageData : pageData?.taxonomy;
    if (!(0, object_js_1.isObjectStrict)(taxObj)) {
        return value;
    }
    const { id, slug, title, contentTypes, usePrimaryContentTypeSlug, isPage } = taxObj;
    const taxonomyTypes = (0, array_js_1.isArrayStrict)(contentTypes) ? contentTypes.map(type => (0, contentType_js_1.normalizeContentType)(type)) : [];
    const primaryTaxonomyType = taxonomyTypes[0];
    value.id = (0, string_js_1.isStringStrict)(id) ? id : '';
    value.slug = (0, string_js_1.isStringStrict)(slug) ? slug : '';
    value.title = (0, string_js_1.isStringStrict)(title) ? title : '';
    value.contentTypes = taxonomyTypes;
    value.primaryContentType = (0, string_js_1.isStringStrict)(primaryTaxonomyType) ? primaryTaxonomyType : '';
    value.usePrimaryContentTypeSlug = (0, boolean_js_1.isBoolean)(usePrimaryContentTypeSlug) ? usePrimaryContentTypeSlug : true;
    value.isPage = (0, boolean_js_1.isBoolean)(isPage) ? isPage : false;
    return value;
};
exports.getTaxonomyInfo = getTaxonomyInfo;
/**
 * Get archive link by content type or taxonomy
 *
 * @param {string} contentType
 * @param {RenderItem} [pageData]
 * @return {ArchiveLink}
 */
const getArchiveLink = (contentType, pageData) => {
    /* Defaults */
    let title = '';
    let slug;
    /* Taxonomy */
    const taxonomyInfo = getTaxonomyInfo(contentType, pageData);
    const { title: taxonomyTitle, isPage: taxonomyIsPage, contentTypes: taxonomyTypes, usePrimaryContentTypeSlug: taxonomyUseTypeSlug } = taxonomyInfo;
    if (contentType === 'term' && taxonomyIsPage) {
        slug = (0, link_js_1.getSlug)({
            slug: '',
            contentType,
            pageData
        });
        title = taxonomyTitle;
    }
    const taxonomyType = taxonomyTypes[0];
    const useArchiveType = taxonomyUseTypeSlug && (0, string_js_1.isStringStrict)(taxonomyType);
    if (contentType === 'taxonomy' && useArchiveType) {
        contentType = taxonomyType;
    }
    if (contentType === 'term' && !taxonomyIsPage && useArchiveType) {
        contentType = taxonomyType;
    }
    /* Archive */
    const archiveInfo = getArchiveInfo(contentType);
    const { id: archiveId, slug: archiveSlug, title: archiveTitle, contentType: archiveType } = archiveInfo;
    if (archiveSlug !== '' && archiveId !== '') {
        slug = (0, link_js_1.getSlug)({
            id: archiveId,
            slug: archiveSlug,
            contentType: archiveType
        });
        const plural = (0, store_js_1.getStoreItem)('archiveMeta')[contentType]?.plural;
        title = (0, string_js_1.isStringStrict)(plural) ? plural : archiveTitle;
    }
    /* Output */
    return {
        title,
        link: (0, string_js_1.isStringStrict)(slug) ? (0, link_js_1.getPermalink)(slug) : ''
    };
};
exports.getArchiveLink = getArchiveLink;
/**
 * Singular and plural labels by content/taxonomy type
 *
 * @param {string} contentType
 * @param {RenderItem} [pageData]
 * @return {ArchiveLabels}
 */
const getArchiveLabels = (contentType, pageData) => {
    /* Defaults */
    let singular = 'Post';
    let plural = 'Posts';
    /* Taxonomy */
    const taxonomyInfo = getTaxonomyInfo(contentType, pageData);
    const { title: taxonomyTitle, isPage: taxonomyIsPage, primaryContentType: taxonomyType, usePrimaryContentTypeSlug: taxonomyUseTypeSlug } = taxonomyInfo;
    if (contentType === 'term' && taxonomyIsPage && (0, string_js_1.isStringStrict)(taxonomyTitle)) {
        return {
            singular: taxonomyTitle,
            plural: taxonomyTitle
        };
    }
    const useArchiveType = taxonomyUseTypeSlug && (0, string_js_1.isStringStrict)(taxonomyType);
    if (contentType === 'taxonomy' && useArchiveType) {
        contentType = taxonomyType;
    }
    if (contentType === 'term' && !taxonomyIsPage && useArchiveType) {
        contentType = taxonomyType;
    }
    if ((0, string_js_1.isStringStrict)(contentType)) {
        contentType = (0, contentType_js_1.normalizeContentType)(contentType);
        const archiveType = (0, store_js_1.getStoreItem)('archiveMeta')[contentType];
        const s = archiveType?.singular;
        const p = archiveType?.plural;
        singular = (0, string_js_1.isStringStrict)(s) ? s : 'Post';
        plural = (0, string_js_1.isStringStrict)(p) ? p : 'Posts';
    }
    return {
        singular,
        plural
    };
};
exports.getArchiveLabels = getArchiveLabels;
