"use strict";
/**
 * Utils - Filter
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.setFilters = exports.resetFilters = exports.applyFilters = exports.removeFilter = exports.addFilter = exports.filters = void 0;
const set_js_1 = require("../set/set.js");
const string_js_1 = require("../string/string.js");
const object_js_1 = require("../object/object.js");
const function_js_1 = require("../function/function.js");
/**
 * Store filter callbacks by name
 *
 * @type {FilterMap}
 */
let filters = new Map([
    ['columnProps', new Set()],
    ['containerProps', new Set()],
    ['fieldProps', new Set()],
    ['formProps', new Set()],
    ['richTextProps', new Set()],
    ['richTextOutput', new Set()],
    ['richTextContentItem', new Set()],
    ['richTextContent', new Set()],
    ['richTextContentOutput', new Set()],
    ['renderItem', new Set()],
    ['renderContent', new Set()],
    ['ajaxResult', new Set()],
    ['cacheData', new Set()],
    ['storeData', new Set()],
    ['contentfulData', new Set()],
    ['wordpressData', new Set()],
    ['localData', new Set()],
    ['allData', new Set()],
    ['slugParts', new Set()],
    ['slug', new Set()]
]);
exports.filters = filters;
/**
 * Add filter to filters map
 *
 * @param {string} name
 * @param {function} filter
 * @return {boolean}
 */
const addFilter = (name, filter) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, function_js_1.isFunction)(filter)) {
        return false;
    }
    if (!(0, set_js_1.isSet)(filters.get(name))) {
        filters.set(name, new Set());
    }
    const filterSet = filters.get(name);
    if ((0, set_js_1.isSet)(filterSet)) {
        filterSet.add(filter);
    }
    return true;
};
exports.addFilter = addFilter;
/**
 * Remove filter from filters map
 *
 * @param {string} name
 * @param {function} filter
 * @return {boolean}
 */
const removeFilter = (name, filter) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, function_js_1.isFunction)(filter)) {
        return false;
    }
    const filterSet = filters.get(name);
    if (!(0, set_js_1.isSet)(filterSet)) {
        return false;
    }
    return filterSet.delete(filter);
};
exports.removeFilter = removeFilter;
/**
 * Call asynchronous functions sequentially
 *
 * @private
 * @param {function[]} callbacks
 * @param {*} value
 * @param {*} [args]
 * @return {*}
 */
const applySequentially = async (callbacks, value, args) => {
    for (const callback of callbacks) {
        value = await callback(value, args);
    }
    return value;
};
/**
 * Update value from callback return values
 *
 * @param {string} name
 * @param {*} value
 * @param {*} [args]
 * @param {boolean} [isAsync]
 * @return {*}
 */
const applyFilters = (name, value, args, isAsync = false) => {
    const filterSet = filters.get(name);
    if (!(0, set_js_1.isSetStrict)(filterSet)) {
        return value;
    }
    const callbacks = [];
    for (const callback of filterSet.values()) {
        if (isAsync) {
            callbacks.push(callback);
        }
        else {
            value = callback(value, args);
        }
    }
    if (isAsync) {
        return applySequentially(callbacks, value, args)
            .then(newValue => newValue)
            .catch(() => value);
    }
    return value;
};
exports.applyFilters = applyFilters;
/**
 * Empty filters map
 *
 * @return {void}
 */
const resetFilters = () => {
    exports.filters = filters = new Map([
        ['columnProps', new Set()],
        ['containerProps', new Set()],
        ['fieldProps', new Set()],
        ['formProps', new Set()],
        ['richTextProps', new Set()],
        ['richTextOutput', new Set()],
        ['richTextContentItem', new Set()],
        ['richTextContent', new Set()],
        ['richTextContentOutput', new Set()],
        ['renderItem', new Set()],
        ['renderContent', new Set()],
        ['ajaxResult', new Set()],
        ['cacheData', new Set()],
        ['storeData', new Set()],
        ['contentfulData', new Set()],
        ['wordpressData', new Set()],
        ['localData', new Set()],
        ['allData', new Set()],
        ['slugParts', new Set()],
        ['slug', new Set()]
    ]);
};
exports.resetFilters = resetFilters;
/**
 * Fill filters map
 *
 * @param {Filters} args
 * @return {boolean}
 */
const setFilters = (args) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return false;
    }
    const newFilters = Object.entries(args);
    if (newFilters.length === 0) {
        return false;
    }
    resetFilters();
    newFilters.forEach(([name, filter]) => {
        if (filter == null) {
            return;
        }
        addFilter(name, filter);
    });
    return true;
};
exports.setFilters = setFilters;
