"use strict";
/**
 * Utils - Image
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getImageMaxWidth = exports.getImageClosestSize = exports.getImage = void 0;
const config_js_1 = require("../../config/config.js");
const string_js_1 = require("../string/string.js");
const array_js_1 = require("../array/array.js");
const number_js_1 = require("../number/number.js");
const object_js_1 = require("../object/object.js");
const dataSource_js_1 = require("../dataSource/dataSource.js");
/**
 * Get responsive image output
 *
 * @param {ImageArgs} args
 * @param {boolean} [returnDetails]
 * @return {ImageReturnType}
 */
const getImage = (args, returnDetails = false) => {
    const { data = undefined, classes = '', attr = '', alt: imageAlt = 'inherit', width = 'auto', height = 'auto', lazy = true, picture = false, quality = 75, source = config_js_1.config.source, maxWidth = 1200, viewportWidth = 100 } = (0, object_js_1.isObjectStrict)(args) ? args : {};
    /* Fallback */
    const fallbackDetails = {
        output: '',
        aspectRatio: 0,
        naturalWidth: 0,
        naturalHeight: 0,
        src: '',
        srcFallback: '',
        srcset: [],
        sizes: ''
    };
    const fallback = returnDetails ? fallbackDetails : '';
    /* Data required */
    if (!(0, object_js_1.isObjectStrict)(data)) {
        return fallback;
    }
    const { path = '', alt: dataAlt = '', width: naturalWidth = 1, height: naturalHeight = 1, format = 'jpg' } = data;
    let { url = config_js_1.config.image.cmsUrl } = data;
    /* Alt */
    let alt = imageAlt === 'inherit' ? dataAlt : imageAlt;
    if (!(0, string_js_1.isStringStrict)(alt)) {
        alt = '';
    }
    /* Source */
    const isLocal = source === 'local';
    const isContentful = dataSource_js_1.dataSource.isContentful(source);
    const isWordpress = dataSource_js_1.dataSource.isWordPress(source);
    /* Local url */
    if (isLocal) {
        if (!(0, string_js_1.isStringStrict)(path)) {
            return fallback;
        }
        url = `${config_js_1.config.image.localUrl}${path}`;
    }
    /* Dimensions */
    const aspectRatio = naturalHeight / naturalWidth;
    const aspectRatioReverse = naturalWidth / naturalHeight;
    let w = naturalWidth;
    let h = naturalHeight;
    if ((0, number_js_1.isNumber)(width)) {
        w = width;
        h = (0, string_js_1.isString)(height) ? w * aspectRatio : height;
    }
    if ((0, number_js_1.isNumber)(height)) {
        h = height;
        w = (0, string_js_1.isString)(width) ? h * aspectRatioReverse : width;
    }
    if (w > maxWidth) {
        w = Math.round(maxWidth);
        h = Math.round(w * aspectRatio);
    }
    /* Src and sizes attributes */
    let src = url;
    let srcFallback = url;
    if (dataSource_js_1.dataSource.isLocal(source)) {
        src = `${url}.webp`;
        srcFallback = `${url}.${format}`;
    }
    if (isContentful) {
        const common = `&q=${quality}&w=${w}&h=${h}`;
        src = `${url}?fm=webp${common}`;
        srcFallback = `${url}?fm=${format}${common}`;
    }
    const sizes = `(min-width: ${w / 16}rem) ${w / 16}rem, ${viewportWidth}vw`;
    const srcsetFallback = [];
    let srcset = [...config_js_1.config.image.sizes];
    if (!srcset.includes(w)) {
        srcset.push(w);
    }
    srcset = srcset.filter(s => s <= w);
    srcset.sort((a, b) => a - b);
    const srcsetSource = [];
    srcset.forEach(s => {
        if (isLocal) {
            const common = `${url}${s !== naturalWidth ? `@${s}` : ''}`;
            srcsetFallback.push(`${common}.${format} ${s}w`);
            srcsetSource.push(`${common}.webp ${s}w`);
        }
        if (isContentful) {
            const common = `&q=${quality}&w=${s}&h=${Math.round(s * aspectRatio)} ${s}w`;
            srcsetFallback.push(`${url}?fm=${format}${common}`);
            srcsetSource.push(`${url}?fm=webp${common}`);
        }
        if (isWordpress) {
            const sizeUrl = data?.sizes?.[s];
            if ((0, string_js_1.isStringStrict)(sizeUrl)) {
                srcsetSource.push(`${sizeUrl} ${s}w`);
            }
        }
    });
    /* Output */
    let sourceOutput = '';
    if (picture) {
        sourceOutput = `<source srcset="${srcsetSource.join(', ')}" sizes="${sizes}" type="image/webp">`;
    }
    let eagerHackOutput = '';
    if (!lazy) {
        eagerHackOutput = `
      <img
        alt=""
        role="presentation"
        aria-hidden="true"
        src="data:image/svg+xml;charset=utf-8,%3Csvg height='${h}' width='${w}' xmlns='http://www.w3.org/2000/svg' version='1.1'%3E%3C/svg%3E" style="pointerEvents: none; position: absolute; top: 0; left: 0; width: 100%; height: 100%"
      >
    `;
    }
    const output = `
    ${eagerHackOutput}
    ${sourceOutput}
    <img
      ${(0, string_js_1.isStringStrict)(classes) ? ` class="${classes}"` : ''}
      alt="${alt}"
      src="${picture ? srcFallback : src}"
      srcset="${picture ? srcsetFallback.join(', ') : srcsetSource.join(', ')}"
      sizes="${sizes}"
      width="${w}"
      height="${h}"
      ${(0, string_js_1.isStringStrict)(attr) ? ` ${attr}` : ''}
      ${lazy ? ' loading="lazy" decoding="async"' : ' loading="eager"'}
    >
  `;
    if (returnDetails) {
        return {
            output,
            aspectRatio,
            naturalWidth,
            naturalHeight,
            src,
            srcFallback,
            srcset: srcsetSource,
            sizes
        };
    }
    return output;
};
exports.getImage = getImage;
/**
 * Get closest value in config sizes
 *
 * @param {number} size
 * @return {number}
 */
const getImageClosestSize = (size) => {
    return [...config_js_1.config.image.sizes].reduce((prev, curr) => {
        return Math.abs(curr - size) <= Math.abs(prev - size) ? curr : prev;
    });
};
exports.getImageClosestSize = getImageClosestSize;
/**
 * Calculate max width from column and container parents
 *
 * @param {ImageMaxWidthArgs} args
 * @return {number}
 */
const getImageMaxWidth = (args) => {
    /* Args must be an object */
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return 0;
    }
    const { parents, widths, maxWidths, breakpoints, source = config_js_1.config.source } = args;
    /* Parents, widths, maxWidths and breakpoints required */
    const required = (0, array_js_1.isArrayStrict)(parents) &&
        (0, object_js_1.isObjectStrict)(widths) &&
        (0, object_js_1.isObjectStrict)(maxWidths) &&
        (0, array_js_1.isArrayStrict)(breakpoints);
    if (!required) {
        return 0;
    }
    /* Widths as floats */
    const w = [];
    /* Max width */
    let m = 0;
    /* Width strings to numbers */
    parents.forEach((parent) => {
        if (!(0, object_js_1.isObjectStrict)(parent)) {
            return;
        }
        const { renderType, args } = parent;
        if (!(0, object_js_1.isObjectStrict)(args)) {
            return;
        }
        if (renderType === 'column') {
            const { width = 'None', widthSmall = 'None', widthMedium = 'None', widthLarge = 'None' } = args;
            w[0] = (0, number_js_1.isNumber)(widths[width]) && widths[width] > 0 ? widths[width] : 1;
            w[1] = (0, number_js_1.isNumber)(widths[widthSmall]) && widths[widthSmall] > 0 ? widths[widthSmall] : w[0];
            w[2] = (0, number_js_1.isNumber)(widths[widthMedium]) && widths[widthMedium] > 0 ? widths[widthMedium] : w[1];
            w[3] = (0, number_js_1.isNumber)(widths[widthLarge]) && widths[widthLarge] > 0 ? widths[widthLarge] : w[2];
        }
        if (renderType === 'container') {
            const { maxWidth = 'None' } = args;
            m = (0, number_js_1.isNumber)(maxWidths[maxWidth]) ? maxWidths[maxWidth] : 0;
        }
    });
    if (w.length === 0 && m === 0) {
        return 0;
    }
    /* Convert to fixed widths */
    const bk = [...breakpoints];
    const calc = [];
    let lastW = 1;
    bk.forEach((b, i) => {
        const wd = w[i];
        if (!(0, number_js_1.isNumber)(wd)) {
            return;
        }
        lastW = wd;
        if (m > 0 && b > m) {
            calc.push(wd * m);
            return;
        }
        calc.push(wd * b);
    });
    if (m > 0) {
        calc.push(lastW * m);
    }
    /* Output */
    const res = Math.max(...calc) * 2;
    if (source === 'local') {
        return getImageClosestSize(res);
    }
    return res;
};
exports.getImageMaxWidth = getImageMaxWidth;
