"use strict";
/**
 * Utils - Image Local
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setLocalImages = void 0;
const node_path_1 = require("node:path");
const promises_1 = require("node:fs/promises");
const sharp_1 = __importDefault(require("sharp"));
const filePath_js_1 = require("../file/filePath.js");
const string_js_1 = require("../string/string.js");
const store_js_1 = require("../../store/store.js");
const config_js_1 = require("../../config/config.js");
/**
 * Get and transform local images (quality and sizes)
 *
 * @return {Promise<PromiseSettledResult<sharp.OutputInfo>[]>}
 */
const setLocalImages = async () => {
    /* Directory paths required */
    const inputDir = config_js_1.config.image.inputDir;
    const outputDir = config_js_1.config.image.outputDir;
    if (!(0, string_js_1.isStringStrict)(inputDir) || !(0, string_js_1.isStringStrict)(outputDir)) {
        throw new Error('No input or output directories');
    }
    await (0, promises_1.mkdir)((0, node_path_1.resolve)(outputDir), { recursive: true });
    /* Process images in input directory */
    const images = [];
    const meta = {};
    for await (const path of (0, filePath_js_1.getFilePaths)(inputDir)) {
        const baseName = (0, node_path_1.basename)(path);
        const ext = (0, node_path_1.extname)(baseName);
        /* Extension required */
        if (!(0, string_js_1.isStringStrict)(ext)) {
            continue;
        }
        /* Base required */
        const [base] = baseName.split(ext);
        if (!(0, string_js_1.isStringStrict)(base?.trim())) {
            continue;
        }
        /* Nested files */
        let [, folders] = (0, node_path_1.dirname)(path).split(`${inputDir}/`);
        if ((0, string_js_1.isStringStrict)(folders) && folders !== baseName) {
            await (0, promises_1.mkdir)((0, node_path_1.resolve)(outputDir, folders), { recursive: true });
        }
        else {
            folders = '';
        }
        /* Image instance */
        const instance = await (0, sharp_1.default)(path);
        /* Store meta data */
        const metadata = await (0, sharp_1.default)(path).metadata();
        const stats = await (0, promises_1.stat)(path);
        const { width = 0, height = 0, format: fileFormat = 'jpeg' } = metadata;
        const id = `${folders !== '' ? `${folders}/` : ''}${base}`;
        const format = ext.replace('.', '');
        meta[id] = {
            path: id,
            name: baseName,
            type: `image/${fileFormat}`,
            format,
            size: stats.size,
            width,
            height
        };
        /* Sizes */
        let sizes = [...config_js_1.config.image.sizes];
        if (!sizes.includes(width)) {
            sizes.push(width);
        }
        sizes = sizes.filter(s => s <= width);
        sizes.forEach((size) => {
            images.push({
                size,
                ext: format,
                path: (0, node_path_1.resolve)(path),
                newPath: (0, node_path_1.resolve)(outputDir, folders, `${base}${size !== width ? `@${size}` : ''}`),
                instance
            });
        });
    }
    if (images.length === 0) {
        throw new Error('No local images to transform');
    }
    (0, store_js_1.setStoreItem)('imageMeta', meta);
    /* Result */
    return await Promise.allSettled(images.map(async (image) => {
        const { size, instance, newPath, ext } = image;
        await instance
            .clone()
            .resize(size)
            .toFile(`${newPath}.${ext}`);
        return await instance
            .clone()
            .resize(size)
            .webp({ quality: config_js_1.config.image.quality })
            .toFile(`${newPath}.webp`);
    }));
};
exports.setLocalImages = setLocalImages;
