"use strict";
/**
 * Utils - Image Remote
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRemoteImages = void 0;
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const node_fs_1 = require("node:fs");
const config_js_1 = require("../../config/config.js");
const string_js_1 = require("../string/string.js");
const array_js_1 = require("../array/array.js");
/**
 * Promisify write function
 *
 * @private
 * @param {string} path
 * @param {Buffer} buffer
 * @return {Promise<void>}
 */
const createFile = async (path, buffer) => {
    return await new Promise((resolve, reject) => {
        (0, node_fs_1.createWriteStream)(path).write(buffer, (error) => {
            if (error instanceof Error) {
                reject(error);
            }
            else {
                resolve();
            }
        });
    });
};
/**
 * Download remote images to local images directory
 *
 * @param {ImageRemote[]} images
 * @return {Promise<PromiseSettledResult<void>[]>}
 */
const getRemoteImages = async (images) => {
    /* Input directory required */
    const inputDir = config_js_1.config.image.inputDir;
    if (!(0, string_js_1.isStringStrict)(inputDir)) {
        throw new Error('No input directory');
    }
    /* Array of image objects required */
    if (!(0, array_js_1.isArrayStrict)(images)) {
        throw new Error('No images array');
    }
    /* Fetch and write images */
    return await Promise.allSettled(images.map(async (image) => {
        const { path, url, ext = 'jpg' } = image;
        if (!(0, string_js_1.isStringStrict)(path) || !(0, string_js_1.isStringStrict)(url) || !(0, string_js_1.isStringStrict)(ext)) {
            return;
        }
        const resp = await fetch(url);
        if (resp.ok) {
            const buffer = await resp.arrayBuffer();
            const fullPath = `${inputDir}/${path}.${ext}`;
            const folders = fullPath.split('/');
            folders.pop();
            await (0, promises_1.mkdir)((0, node_path_1.resolve)(folders.join('/')), { recursive: true });
            return await createFile((0, node_path_1.resolve)(fullPath), Buffer.from(buffer));
        }
    }));
};
exports.getRemoteImages = getRemoteImages;
