"use strict";
/**
 * Utils - Link
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPermalink = exports.getLink = exports.getSlug = void 0;
const number_js_1 = require("../number/number.js");
const object_js_1 = require("../object/object.js");
const string_js_1 = require("../string/string.js");
const filter_js_1 = require("../filter/filter.js");
const archive_js_1 = require("../archive/archive.js");
const store_js_1 = require("../../store/store.js");
const config_js_1 = require("../../config/config.js");
/**
 * Recurse to get ascendents
 *
 * @private
 * @param {string} id
 * @param {string} contentType
 * @param {LinkSlugParent[]} parents
 * @return {void}
 */
const getParentSlug = (id, contentType, parents) => {
    const storeParents = (0, store_js_1.getStoreItem)('parents');
    const parent = storeParents[contentType]?.[id];
    if (parent != null) {
        parents.unshift({ ...parent, contentType });
        getParentSlug(parent.id, contentType, parents);
    }
};
/**
 * Get slug with archive/taxonomy base and parents
 *
 * @param {LinkSlugArgs} args
 * @return {LinkSlugReturnType}
 */
const getSlug = (args, returnParents = false) => {
    let { id = '', slug = '', page = 0, contentType = 'page', pageData = undefined } = (0, object_js_1.isObjectStrict)(args) ? args : {};
    /* Types slugs */
    let typeSlug = '';
    let taxSlug = '';
    let parentSlugs = '';
    /* Parts */
    let parts = [];
    /* Config types */
    const { hierarchicalTypes, typeInSlug, localeInSlug } = config_js_1.config;
    /* Locale */
    const pageLocale = pageData?.locale;
    const locale = (0, string_js_1.isStringStrict)(pageLocale) ? localeInSlug[pageLocale] : '';
    const hasLocale = (0, string_js_1.isStringStrict)(locale);
    /* Term/taxonomy */
    const taxonomyInfo = (0, archive_js_1.getTaxonomyInfo)(contentType, pageData);
    const isTaxonomy = contentType === 'taxonomy';
    const isTerm = contentType === 'term';
    const { id: taxonomyId, slug: taxonomySlug, title: taxonomyTitle, primaryContentType: taxonomyType, usePrimaryContentTypeSlug: taxonomyUseTypeSlug, isPage: taxonomyIsPage } = taxonomyInfo;
    /* Archive */
    const archiveType = (0, string_js_1.isStringStrict)(taxonomyType) ? taxonomyType : contentType;
    const archiveInfo = (0, archive_js_1.getArchiveInfo)(archiveType);
    const { id: archiveId, slug: archiveSlug, title: archiveTitle, contentType: archiveContentType } = archiveInfo;
    /* Archive and/or taxonomy parent */
    const archiveParents = [];
    if (archiveSlug !== '' && archiveId !== '') {
        typeSlug = archiveSlug;
        archiveParents.push({
            contentType: archiveContentType,
            title: archiveTitle,
            slug: archiveSlug,
            id: archiveId
        });
    }
    if ((isTaxonomy || isTerm) && !taxonomyUseTypeSlug) {
        typeSlug = '';
    }
    if (isTerm && taxonomySlug !== '' && taxonomyId !== '') {
        taxSlug = taxonomySlug;
        if (taxonomyIsPage) {
            archiveParents.push({
                contentType: 'taxonomy',
                title: taxonomyTitle,
                slug: taxonomySlug,
                id: taxonomyId
            });
        }
    }
    /* Content type */
    const contentTypeInSlug = typeInSlug[archiveType];
    if ((0, string_js_1.isString)(contentTypeInSlug)) {
        typeSlug = contentTypeInSlug;
    }
    if ((0, object_js_1.isObjectStrict)(contentTypeInSlug) && hasLocale) {
        const localizedTypeSlug = contentTypeInSlug[locale];
        if ((0, string_js_1.isString)(localizedTypeSlug)) {
            typeSlug = localizedTypeSlug;
        }
    }
    /* Page parents */
    const isHierarchical = hierarchicalTypes.includes(contentType);
    let parents = [];
    let hasParents = false;
    getParentSlug(isHierarchical ? id : archiveId, isHierarchical ? contentType : archiveContentType, parents);
    if (parents.length > 0) {
        parentSlugs = parents.map(({ slug }) => slug).join('/');
        hasParents = true;
    }
    if (isHierarchical) {
        parents = [...archiveParents, ...parents];
    }
    else {
        parents = [...parents, ...archiveParents];
    }
    /* Index */
    const isIndex = slug === 'index';
    if (isIndex && returnParents === false) {
        slug = '';
    }
    /* Filter parts */
    if (hasLocale) {
        parts.push(locale);
    }
    if (hasParents && !isHierarchical) {
        parts.push(parentSlugs);
    }
    if (typeSlug !== '') {
        parts.push(typeSlug);
    }
    if (taxSlug !== '') {
        parts.push(taxSlug);
    }
    if (hasParents && isHierarchical) {
        parts.push(parentSlugs);
    }
    if ((0, string_js_1.isStringStrict)(slug) && !isIndex) {
        parts.push(slug);
    }
    parts = (0, filter_js_1.applyFilters)('slugParts', parts, args);
    /* Slug */
    let fullSlug = `${parts.length > 0 ? parts.join('/') : ''}${(0, number_js_1.isNumber)(page) && page > 1 ? `/?page=${page}` : ''}`;
    fullSlug = (0, filter_js_1.applyFilters)('slug', fullSlug, args);
    /* Parents and slug return */
    if (returnParents === true) {
        const res = {
            slug: fullSlug,
            parents
        };
        return res;
    }
    /* Slug return */
    return fullSlug;
};
exports.getSlug = getSlug;
/**
 * Get absolute or relative url
 *
 * @param {string} slug
 * @param {boolean} trailingSlash
 * @return {string}
 */
const getPermalink = (slug = '', trailingSlash = true) => {
    let url = '/';
    if (config_js_1.config.env.prod) {
        url = config_js_1.config.env.prodUrl;
    }
    if (slug === '/') {
        return url;
    }
    return `${url}${slug}${slug !== '' && trailingSlash ? '/' : ''}`;
};
exports.getPermalink = getPermalink;
/**
 * Get permalink from external or internal source
 *
 * @param {InternalLink} [internalLink]
 * @param {string} [externalLink]
 * @return {string}
 */
const getLink = (internalLink, externalLink) => {
    if ((0, object_js_1.isObjectStrict)(internalLink)) {
        const slug = internalLink.slug;
        const res = getSlug({
            id: internalLink.id,
            contentType: internalLink.contentType,
            pageData: internalLink,
            slug: (0, string_js_1.isString)(slug) ? slug : ''
        });
        return getPermalink(res);
    }
    return (0, string_js_1.isStringStrict)(externalLink) ? externalLink : '';
};
exports.getLink = getLink;
