"use strict";
/**
 * Utils - Script Style
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.outputStyles = exports.outputScripts = exports.addStyle = exports.addScript = exports.styles = exports.scripts = void 0;
const array_js_1 = require("../array/array.js");
const string_js_1 = require("../string/string.js");
const config_js_1 = require("../../config/config.js");
/**
 * Scripts data per render item
 *
 * @type {Scripts}
 */
const scripts = {
    deps: new Map(),
    item: new Map(),
    build: new Map(),
    meta: {}
};
exports.scripts = scripts;
/**
 * Styles data per render item
 *
 * @type {Styles}
 */
const styles = {
    deps: new Map(),
    item: new Map(),
    build: new Map()
};
exports.styles = styles;
/**
 * Add style or script path and dependencies to item and build maps
 *
 * @private
 * @param {string} type - styles | scripts
 * @param {string} path
 * @param {string[]} [deps]
 * @return {boolean}
 */
const addScriptStyle = (type, path, deps = []) => {
    /* Path required */
    if (!(0, string_js_1.isStringStrict)(path)) {
        return false;
    }
    /* Input and output dir */
    const isScripts = type === 'scripts';
    const assets = isScripts ? scripts : styles;
    const ext = isScripts ? 'js' : 'scss';
    const { inputDir, outputDir } = config_js_1.config[type];
    const input = `${inputDir}/${path}.${ext}`;
    const output = `${outputDir}/${path}`;
    /* Add to item and build */
    assets.item.set(output, input);
    assets.build.set(output, input);
    /* Add deps */
    if ((0, array_js_1.isArrayStrict)(deps)) {
        deps.forEach((dep) => {
            const depInput = `${inputDir}/${dep}.${ext}`;
            const depOutput = `${outputDir}/${dep}`;
            if (!assets.deps.has(output)) {
                assets.deps.set(output, new Set());
            }
            assets.deps.get(output)?.add(depOutput);
            assets.item.set(depOutput, depInput);
            assets.build.set(depOutput, depInput);
        });
    }
    /* Append success */
    return true;
};
/**
 * Add script path and dependencies to item and build maps
 *
 * @param {string} path
 * @param {string[]} [deps]
 * @return {boolean}
 */
const addScript = (path, deps = []) => {
    return addScriptStyle('scripts', path, deps);
};
exports.addScript = addScript;
/**
 * Add style path and dependencies to item and build maps
 *
 * @param {string} path
 * @param {string[]} [deps]
 * @return {boolean}
 */
const addStyle = (path, deps = []) => {
    return addScriptStyle('styles', path, deps);
};
exports.addStyle = addStyle;
/**
 * Output script or stylesheet elements
 *
 * @private
 * @param {string} type - styles | scripts
 * @param {string} link
 * @return {string}
 */
const outputScriptsStyles = (type, link) => {
    /* Link is required */
    if (!(0, string_js_1.isStringStrict)(link)) {
        return '';
    }
    /* Vars */
    const { item, deps } = type === 'scripts' ? scripts : styles;
    /* Check if scripts exist */
    if (item.size === 0) {
        return '';
    }
    /* Sort by dependencies */
    const itemOut = Array.from(item.keys());
    itemOut.sort((a, b) => {
        const aIsDep = deps.get(b)?.has(a);
        const bIsDep = deps.get(a)?.has(b);
        if (aIsDep === true) {
            return -1;
        }
        if (bIsDep === true) {
            return 1;
        }
        return 0;
    });
    /* Output */
    let output = '';
    itemOut.forEach((out) => {
        if (type === 'scripts') {
            output += `<script type="module" src="${link}${out}.js"></script>`;
        }
        if (type === 'styles') {
            output += `<link rel="stylesheet" href="${link}${out}.css" media="all">`;
        }
    });
    return output;
};
/**
 * Output script elements
 *
 * @param {string} link
 * @return {string}
 */
const outputScripts = (link) => {
    return outputScriptsStyles('scripts', link);
};
exports.outputScripts = outputScripts;
/**
 * Output stylesheet elements
 *
 * @param {string} link
 * @return {string}
 */
const outputStyles = (link) => {
    return outputScriptsStyles('styles', link);
};
exports.outputStyles = outputStyles;
