"use strict";
/**
 * WordPress - Data Normal
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.taxonomiesById = exports.normalizeWordPressData = void 0;
const block_serialization_spec_parser_1 = require("@wordpress/block-serialization-spec-parser");
const objectUtils_js_1 = require("../utils/object/objectUtils.js");
const link_js_1 = require("../utils/link/link.js");
const string_js_1 = require("../utils/string/string.js");
const array_js_1 = require("../utils/array/array.js");
const object_js_1 = require("../utils/object/object.js");
const number_js_1 = require("../utils/number/number.js");
const config_js_1 = require("../config/config.js");
/**
 * Camel case version of prop names
 *
 * @private
 * @type {Object<string, string>}
 */
const camelCaseKeys = {
    date_gmt: 'dateGmt',
    modified_gmt: 'modifiedGmt',
    featured_media: 'featuredMedia',
    menu_order: 'menuOrder',
    comment_status: 'commentStatus',
    ping_status: 'pingStatus',
    class_list: 'classList',
    attr_title: 'attrTitle',
    type_label: 'typeLabel',
    object_id: 'objectId',
    alt_text: 'altText',
    media_type: 'mediaType',
    media_details: 'details',
    mime_type: 'mimeType',
    source_url: 'sourceUrl'
};
/**
 * Properties to exclude fro item
 *
 * @private
 * @type {string[]}
 */
const excludeProps = [
    '_links',
    'auto_add',
    'rest_base',
    'rest_namespace'
];
/**
 * Menu items grouped by menu id
 *
 * @private
 * @type {Map<string, WordPressDataMenuChild[]>}
 */
const menusById = new Map();
/**
 * Taxonomies grouped by id
 *
 * @type {Map<string, RenderItem>}
 */
const taxonomiesById = new Map();
exports.taxonomiesById = taxonomiesById;
/**
 * Taxonomy from taxonomies given id
 *
 * @private
 * @param {string} id
 * @return {Taxonomy}
 */
const getTaxonomy = (id) => {
    const taxonomy = taxonomiesById.get(id);
    const { title = '', slug = '', contentTypes = [] } = (0, object_js_1.isObjectStrict)(taxonomy) ? taxonomy : {};
    return {
        id,
        title,
        slug,
        contentTypes
        // usePrimaryContentTypeSlug
    };
};
/**
 * Remove #text tags and set attr
 *
 * @private
 * @param {WordPressDataRichText} item
 * @return {RenderItem}
 */
const normalizeRichText = (item) => {
    const { tag, content, attrs } = item;
    if (tag === '#text') {
        item.tag = '';
    }
    if ((0, object_js_1.isObjectStrict)(attrs)) {
        const attr = [];
        for (const [k, v] of Object.entries(attrs)) {
            if (k === 'href') {
                item.link = v;
                continue;
            }
            attr.push(`${k}="${v}"`);
        }
        item.attr = attr.join(' ');
        item.attrs = undefined;
    }
    if ((0, array_js_1.isArrayStrict)(content)) {
        const newContent = content.map((c) => {
            return normalizeRichText(c);
        });
        item.content = newContent;
    }
    return item;
};
/**
 * Reduce file props
 *
 * @private
 * @param {WordPressDataFile} file
 * @return {RenderFile}
 */
const normalizeFile = (file) => {
    const { url, filename, alt, width, height, filesizeInBytes, subtype, mime, sizes } = file;
    let s;
    if ((0, object_js_1.isObjectStrict)(sizes)) {
        s = {};
        for (const [, value] of Object.entries(sizes)) {
            let { width, url, source_url: src } = value;
            if ((0, string_js_1.isStringStrict)(src)) {
                url = src;
            }
            if (!(0, number_js_1.isNumber)(width) || !(0, string_js_1.isStringStrict)(url)) {
                continue;
            }
            s[width] = url;
        }
    }
    return {
        url: (0, string_js_1.isString)(url) ? url : '',
        name: (0, string_js_1.isString)(filename) ? filename : '',
        alt: (0, string_js_1.isString)(alt) ? alt : '',
        width: (0, number_js_1.isNumber)(width) ? width : 0,
        height: (0, number_js_1.isNumber)(height) ? height : 0,
        size: (0, number_js_1.isNumber)(filesizeInBytes) ? filesizeInBytes : 0,
        format: subtype === 'jpeg' ? 'jpg' : subtype,
        type: mime,
        sizes: s
    };
};
/**
 * Link embeds to item parent, author, featured media or term
 *
 * @private
 * @param {WordPressDataEmbedded} value
 * @param {WordPressDataItem} item
 * @param {RenderItem} newItem
 * @return {void}
 */
const normalizeEmbedded = (value, item, newItem) => {
    (0, objectUtils_js_1.getObjectKeys)(value).forEach((k) => {
        const embeds = value[k];
        /* Must be array */
        if (!(0, array_js_1.isArrayStrict)(embeds)) {
            return;
        }
        /* Author */
        if (k === 'author') {
            embeds.forEach((embed) => {
                if (!(0, object_js_1.isObjectStrict)(embed)) {
                    return;
                }
                const { id, name, url, description, link, slug } = embed;
                if (item.author !== id) {
                    return;
                }
                newItem.author = {
                    id: id.toString(),
                    name,
                    url,
                    description,
                    link,
                    slug
                };
            });
        }
        /* Parent */
        if (k === 'up') {
            embeds.forEach((embed) => {
                if (!(0, object_js_1.isObjectStrict)(embed)) {
                    return;
                }
                const { id, title: parentTitle, name, type, taxonomy, slug } = embed;
                if (item.parent !== id) {
                    return;
                }
                let title = '';
                if ((0, string_js_1.isString)(parentTitle?.rendered)) {
                    title = parentTitle.rendered;
                }
                if ((0, string_js_1.isString)(name)) {
                    title = name;
                }
                newItem.parent = {
                    id: id.toString(),
                    contentType: (0, string_js_1.isString)(taxonomy) ? 'term' : type,
                    title,
                    slug
                };
            });
        }
        /* Featured media */
        if (k === 'wp:featuredmedia') {
            embeds.forEach((embed) => {
                if (!(0, object_js_1.isObjectStrict)(embed)) {
                    return;
                }
                const { id, source_url: url, media_details: details, mime_type: mimeType, alt_text: alt } = embed;
                if (!(0, object_js_1.isObjectStrict)(details)) {
                    return;
                }
                const { file, filesize, width, height, sizes } = details;
                if (item.featured_media !== id) {
                    return;
                }
                newItem.featuredMedia = normalizeFile({
                    url,
                    filename: file.split('/').pop(),
                    alt,
                    width,
                    height,
                    filesizeInBytes: filesize,
                    subtype: mimeType.split('/').pop(),
                    mime: mimeType,
                    sizes
                });
            });
        }
        /* Term */
        if (k === 'wp:term') {
            embeds.forEach((embed) => {
                if (!(0, array_js_1.isArrayStrict)(embed)) {
                    return;
                }
                embed.forEach((e) => {
                    if (!(0, object_js_1.isObjectStrict)(e)) {
                        return;
                    }
                    const { id, link, name, slug, taxonomy } = e;
                    let taxonomyLookup = taxonomy;
                    if (taxonomy === 'category') {
                        taxonomyLookup = 'categories';
                    }
                    if (taxonomy === 'post_tag') {
                        taxonomyLookup = 'tags';
                    }
                    const itemTaxonomy = item[taxonomyLookup];
                    if (!(0, array_js_1.isArrayStrict)(itemTaxonomy)) {
                        return;
                    }
                    newItem[taxonomyLookup] = itemTaxonomy.map((taxonomyId) => {
                        if (taxonomyId !== id) {
                            return taxonomyId;
                        }
                        return {
                            id: id.toString(),
                            link,
                            title: name,
                            slug,
                            contentType: 'term',
                            taxonomy: getTaxonomy(taxonomy)
                        };
                    });
                });
            });
        }
    });
};
/**
 * Convert blocks to flatter props
 *
 * @private
 * @param {Block[]} blocks
 * @return {RenderItem[]}
 */
const normalizeBlocks = (blocks) => {
    const newItems = [];
    /* Recurse */
    blocks.forEach((block) => {
        const { blockName, attrs, innerBlocks } = block;
        const contentType = blockName;
        if (!(0, string_js_1.isStringStrict)(contentType)) {
            return;
        }
        const { contentIsAttribute, attributeIsItem } = attrs;
        const attrItemArr = ((0, string_js_1.isStringStrict)(attributeIsItem) ? attributeIsItem : '').split(',');
        const attrItemExists = attrItemArr.length > 0;
        for (const [key, value] of Object.entries(attrs)) {
            if (!(0, object_js_1.isObjectStrict)(value)) {
                continue;
            }
            if (attrItemExists && attrItemArr.includes(key)) {
                const itemValue = normalizeItem(value);
                itemValue.content = undefined;
                attrs[key] = itemValue;
            }
            if ((0, string_js_1.isStringStrict)(value.mime)) {
                attrs[key] = normalizeFile(value);
            }
        }
        let newItem = {
            contentType,
            ...attrs
        };
        const renderType = config_js_1.config.renderTypes[contentType];
        if ((0, string_js_1.isString)(renderType)) {
            newItem.renderType = renderType;
        }
        if (renderType === 'richText') {
            newItem = normalizeRichText(newItem);
        }
        if ((0, array_js_1.isArrayStrict)(innerBlocks)) {
            const contentAttr = (0, string_js_1.isStringStrict)(contentIsAttribute) ? contentIsAttribute : 'content';
            newItem[contentAttr] = normalizeBlocks(innerBlocks);
        }
        return newItems.push(newItem);
    });
    /* Output */
    return newItems;
};
/**
 * Transform item props to flatter structure
 *
 * @private
 * @param {WordPressDataItem} item
 * @return {RenderItem}
 */
const normalizeItem = (item) => {
    const newItem = {};
    /* Loop item */
    for (const [key, value] of Object.entries(item)) {
        /* Skip prop */
        if (excludeProps.includes(key)) {
            continue;
        }
        /* Value */
        let val = value;
        /* Key */
        let k = camelCaseKeys[key] != null ? camelCaseKeys[key] : key;
        /* Check types */
        const isObj = (0, object_js_1.isObjectStrict)(value);
        const isStr = (0, string_js_1.isStringStrict)(value);
        /* Id */
        if (key === 'id' || k === 'objectId') {
            val = value?.toString();
        }
        /* Content type */
        if (key === 'type' && isStr) {
            newItem.contentType = value;
            if ((0, string_js_1.isString)(config_js_1.config.renderTypes[value])) {
                newItem.renderType = config_js_1.config.renderTypes[value];
            }
            continue;
        }
        if (key === 'types') {
            k = 'contentTypes';
        }
        /* Flatten rendered */
        if (isObj) {
            const { rendered } = value;
            if ((0, string_js_1.isString)(rendered)) {
                val = rendered;
            }
        }
        /* Name as title */
        if (key === 'name') {
            k = 'title';
        }
        /* Embedded */
        if (key === '_embedded' && isObj) {
            normalizeEmbedded(value, item, newItem);
            continue;
        }
        /* Excerpt, content and taxonomy */
        if ((0, string_js_1.isStringStrict)(val)) {
            if (key === 'excerpt') {
                val = val.replace(/<[^>]*>|\[.*?\]/g, '').trim();
            }
            if (key === 'content') {
                const normalVal = normalizeBlocks((0, block_serialization_spec_parser_1.parse)(val));
                val = normalVal.length > 0 ? normalVal : val;
            }
            if (key === 'taxonomy') {
                newItem.contentType = 'term';
                val = getTaxonomy(val);
            }
        }
        /* Meta */
        if (key === 'meta' && isObj) {
            for (const [metaKey, metaValue] of Object.entries(val)) {
                newItem[metaKey] = metaValue;
            }
            continue;
        }
        /* Media details */
        if (k === 'details' && isObj) {
            const valObj = val;
            const valFull = valObj.sizes.full;
            val = normalizeFile({
                url: item?.source_url,
                filename: item?.source_url?.split('/').pop(),
                alt: item?.alt_text,
                width: valFull?.width,
                height: valFull?.height,
                filesizeInBytes: valObj?.filesize,
                subtype: item?.mime_type?.split('/')[1],
                mime: item?.mime_type,
                sizes: valObj.sizes
            });
        }
        /* Set value */
        newItem[k] = val;
    }
    /* Output */
    return newItem;
};
/**
 * Transform wordpress menu items into navigation item objects
 *
 * @private
 * @param {WordPressDataMenuItem[]} items
 * @return {NavigationItem[]}
 */
const normalizeWordPressMenuItems = (items) => {
    const itemsObj = Object.fromEntries(items.map(item => [item.id, item]));
    menusById.clear();
    items.forEach((item) => {
        const { id = '', title = '', menuOrder = 0, menus = 0, parent } = item;
        const hasMenuOrder = (0, number_js_1.isNumber)(menuOrder);
        if (hasMenuOrder && (0, object_js_1.isObjectStrict)(itemsObj[parent])) {
            if (itemsObj[parent]?.children == null) {
                itemsObj[parent].children = [];
            }
            itemsObj[parent].children.push({ id, menuOrder, title });
        }
        if (hasMenuOrder && (0, number_js_1.isNumber)(menus) && parent === 0) {
            const menuId = menus.toString();
            if (menusById.get(menuId) == null) {
                menusById.set(menuId, []);
            }
            menusById.get(menuId)?.push({ id, menuOrder, title });
        }
    });
    const newItems = [];
    for (const [id, obj] of Object.entries(itemsObj)) {
        const { url = '', title = '', attrTitle = '', description = '', contentType = '', children = [], object = '', objectId = '', target = '', classes = [], xfn = [], meta = [] } = obj;
        if (!(0, string_js_1.isStringStrict)(url)) {
            continue;
        }
        const isCustom = contentType === 'custom';
        const newItem = {
            id,
            title,
            attrTitle,
            description,
            contentType: 'navigationItem',
            target,
            classes,
            xfn,
            meta
        };
        if (!isCustom && (0, string_js_1.isStringStrict)(object) && (0, string_js_1.isStringStrict)(objectId)) {
            const slug = url.split('/').filter(Boolean).pop();
            const isTerm = contentType === 'taxonomy';
            const internalLink = {
                contentType: isTerm ? 'term' : object,
                id: objectId,
                slug
            };
            if (isTerm) {
                internalLink.taxonomy = getTaxonomy(object);
            }
            newItem.link = url;
            newItem.internalLink = internalLink;
        }
        if (isCustom) {
            const permalink = (0, link_js_1.getPermalink)();
            const isExternal = permalink !== '' ? !url.startsWith(permalink) : true;
            if (isExternal) {
                newItem.externalLink = url;
            }
        }
        if ((0, array_js_1.isArrayStrict)(children)) {
            newItem.children = children.sort((a, b) => a.menuOrder - b.menuOrder);
        }
        newItems.push(newItem);
    }
    return newItems;
};
/**
 * Transform wordpress menus into navigation item objects
 *
 * @private
 * @param {WordPressDataMenu[]} menus
 * @return {Navigation[]}
 */
const normalizeWordPressMenus = (menus) => {
    const newMenus = [];
    menus.forEach((menu) => {
        const { id = '', title = '', description = '', locations = [], meta = [] } = menu;
        const newMenu = {
            id,
            title,
            description,
            meta,
            location: locations,
            items: menusById.get(id)?.sort((a, b) => a.menuOrder - b.menuOrder) ?? []
        };
        newMenus.push(newMenu);
    });
    return newMenus;
};
/**
 * Transform wordpress data into simpler objects
 *
 * @param {WordPressDataItem[]} data
 * @param {string} [route]
 * @param {RenderItem[]} [_newData]
 * @return {RenderItem[]}
 */
const normalizeWordPressData = (data, route = '', _newData = []) => {
    /* Data must be array */
    if (!(0, array_js_1.isArrayStrict)(data)) {
        return [];
    }
    /* Taxonomies */
    if (route === 'taxonomies') {
        data = Object.entries(data[0]).map(([key, value]) => ({
            id: key,
            type: 'taxonomy',
            ...value
        }));
    }
    /* Recurse data */
    data.forEach((item) => {
        if (!(0, object_js_1.isObjectStrict)(item)) {
            return;
        }
        _newData.push(normalizeItem(item));
    });
    /* Customize by route */
    if (route === 'menu-items') {
        _newData = normalizeWordPressMenuItems(_newData);
    }
    if (route === 'menus') {
        _newData = normalizeWordPressMenus(_newData);
    }
    if (route === 'taxonomies') {
        taxonomiesById.clear();
        _newData.forEach(item => {
            const { id = '' } = item;
            taxonomiesById.set(id, item);
        });
    }
    /* Output */
    return _newData;
};
exports.normalizeWordPressData = normalizeWordPressData;
