/**
 * Store
 */
import { normalizeContentType } from '../utils/contentType/contentType.js';
import { isObject, isObjectStrict } from '../utils/object/object.js';
import { isStringStrict } from '../utils/string/string.js';
import { isArrayStrict } from '../utils/array/array.js';
import { getJsonFile } from '../utils/json/json.js';
import { config } from '../config/config.js';
/**
 * Directory to write store files to
 *
 * @type {string}
 */
let storeDir = 'lib/store';
/**
 * Default store object
 *
 * @type {Store}
 */
const defaultStore = {
    slugs: {},
    parents: {},
    navigations: [],
    navigationItems: [],
    formMeta: {},
    archiveMeta: {},
    imageMeta: {}
};
/**
 * Store options
 *
 * @type {Store}
 */
let store = {
    ...defaultStore
};
/**
 * Update default store with user options
 *
 * @param {Store} args
 * @param {string} [dir]
 * @return {Store}
 */
const setStore = (args, dir = 'lib/store') => {
    if (!isObjectStrict(args)) {
        return store;
    }
    store = { ...defaultStore, ...args };
    if (isStringStrict(dir)) {
        storeDir = dir;
    }
    return store;
};
/**
 * Update individual store property
 *
 * @param {string} prop
 * @param {object} value
 * @param {string} [subProp]
 * @return {boolean}
 */
const setStoreItem = (prop, value, subProp) => {
    if (!isStringStrict(prop) || !isObject(value) || store[prop] == null) {
        return false;
    }
    if (isStringStrict(subProp)) {
        store[prop][subProp] = value;
    }
    else {
        store[prop] = value;
    }
    return true;
};
/**
 * Set serverless or build time data (navigations, archive meta, parents)
 *
 * @param {RenderAllData} allData
 * @param {boolean} [isServerless]
 * @return {Promise<boolean>}
 */
const setStoreData = async (allData, isServerless = false) => {
    /* Serverless all data */
    if (isServerless) {
        for (const key of Object.keys(store)) {
            if (key === 'slugs') { // Skip slugs as set in data fetch
                continue;
            }
            const data = await getJsonFile(key);
            if (data != null) {
                store[key] = data;
            }
        }
        return true;
    }
    /* Data must be object */
    if (!isObjectStrict(allData)) {
        return false;
    }
    const { navigation, navigationItem, content } = allData;
    /* Build time navigation data */
    store.navigations = isArrayStrict(navigation) ? navigation : [];
    store.navigationItems = isArrayStrict(navigationItem) ? navigationItem : [];
    /* Build time archive meta and parent data */
    const data = {
        ...allData,
        ...content
    };
    config.hierarchicalTypes.forEach(type => {
        const items = data?.[type];
        if (!isArrayStrict(items)) {
            return;
        }
        items.forEach(item => {
            if (!isObjectStrict(item)) {
                return;
            }
            /* Props */
            const { id, parent, archive, slug, title } = item;
            /* Id required */
            if (!isStringStrict(id)) {
                return;
            }
            /* Archive */
            const archiveType = normalizeContentType(archive);
            if (isStringStrict(archiveType)) {
                const archiveObj = isObjectStrict(store.archiveMeta[archiveType]) ? store.archiveMeta[archiveType] : {};
                store.archiveMeta[archiveType] = {
                    id,
                    slug,
                    title,
                    contentType: type,
                    ...archiveObj
                };
            }
            /* Parent */
            if (isObjectStrict(parent)) {
                const parentSlug = parent.slug;
                const parentTitle = parent.title;
                const parentId = parent.id;
                if (isStringStrict(parentSlug) && isStringStrict(parentTitle) && isStringStrict(parentId)) {
                    if (store.parents[type] == null) {
                        store.parents[type] = {};
                    }
                    store.parents[type][id] = {
                        id: parentId,
                        slug: parentSlug,
                        title: parentTitle
                    };
                }
            }
        });
    });
    return true;
};
/**
 * Get individual store property
 *
 * @param {string} prop
 * @return {object}
 */
const getStoreItem = (prop) => {
    return store[prop];
};
/**
 * Set and get serverless store property
 *
 * @param {string} prop
 * @return {object}
 */
const fetchStoreItem = async (prop) => {
    const data = await getJsonFile(prop);
    if (data != null) {
        store[prop] = data;
    }
    return store[prop];
};
/* Exports */
export { store, storeDir, setStore, setStoreData, fetchStoreItem, setStoreItem, getStoreItem };
