/**
 * Utils - Action
 */
import { isSet, isSetStrict } from '../set/set.js';
import { isStringStrict } from '../string/string.js';
import { isObjectStrict } from '../object/object.js';
import { isFunction } from '../function/function.js';
/**
 * Action callbacks by name
 *
 * @type {ActionMap}
 */
let actions = new Map([
    ['renderStart', new Set()],
    ['renderEnd', new Set()],
    ['renderItemStart', new Set()],
    ['renderItemEnd', new Set()]
]);
/**
 * Add action to action map
 *
 * @param {string} name
 * @param {function} action
 * @return {boolean}
 */
const addAction = (name, action) => {
    if (!isStringStrict(name) || !isFunction(action)) {
        return false;
    }
    if (!isSet(actions.get(name))) {
        actions.set(name, new Set());
    }
    const actionSet = actions.get(name);
    if (isSet(actionSet)) {
        actionSet.add(action);
    }
    return true;
};
/**
 * Remove action from actions map
 *
 * @param {string} name
 * @param {function} action
 * @return {boolean}
 */
const removeAction = (name, action) => {
    if (!isStringStrict(name) || !isFunction(action)) {
        return false;
    }
    const actionSet = actions.get(name);
    if (!isSet(actionSet)) {
        return false;
    }
    return actionSet.delete(action);
};
/**
 * Call asynchronous functions sequentially
 *
 * @private
 * @param {function[]} callbacks
 * @param {*} [args]
 * @return {void}
 */
const doSequentially = async (callbacks, args) => {
    for (const callback of callbacks) {
        await callback(args);
    }
};
/**
 * Run callback functions from actions map
 *
 * @param {string} name
 * @param {*} [args]
 * @param {boolean} [isAsync]
 * @return {*}
 */
const doActions = (name, args, isAsync = false) => {
    const actionSet = actions.get(name);
    if (!isSetStrict(actionSet)) {
        return undefined;
    }
    const callbacks = [];
    for (const callback of actionSet.values()) {
        if (isAsync) {
            callbacks.push(callback);
        }
        else {
            callback(args);
        }
    }
    if (isAsync) {
        doSequentially(callbacks, args)
            .then(result => result)
            .catch(() => false);
    }
    return undefined;
};
/**
 * Empty actions map
 *
 * @return {void}
 */
const resetActions = () => {
    actions = new Map([
        ['renderStart', new Set()],
        ['renderEnd', new Set()],
        ['renderItemStart', new Set()],
        ['renderItemEnd', new Set()]
    ]);
};
/**
 * Fill actions map
 *
 * @param {Actions} args
 * @return {boolean}
 */
const setActions = (args) => {
    if (!isObjectStrict(args)) {
        return false;
    }
    const newActions = Object.entries(args);
    if (newActions.length === 0) {
        return false;
    }
    resetActions();
    newActions.forEach(([name, action]) => {
        if (action == null) {
            return;
        }
        addAction(name, action);
    });
    return true;
};
/* Exports */
export { actions, addAction, removeAction, doActions, resetActions, setActions };
